# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from lxml import etree as etree
from lxml.etree import SubElement
from flask import current_app as app
from superdesk.publish.formatters import Formatter
from superdesk.errors import FormatterError
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO, FORMAT, FORMATS
from apps.archive.common import get_utc_schedule
from superdesk.text_utils import get_text


class EraseElement(Exception):
    """Remove entirely element"""


class NITFFormatter(Formatter):
    """NITF Formatter for Superdesk

    Format items to `NITF <https://iptc.org/standards/nitf/>`_ version *3.6*.
    """

    XML_ROOT = '<?xml version="1.0"?>'

    _message_attrib = {'version': "-//IPTC//DTD NITF 3.6//EN"}

    _schema_uri = 'http://www.iptc.org/std/NITF/3.6/specification'
    _schema_ref = 'http://www.iptc.org/std/NITF/3.6/specification/nitf-3-6.xsd'
    _debug_message_extra = {
        'schemaLocation': '{} {}'.format(_schema_uri, _schema_ref)}
    NITF_COMMON_ATTR = ('id', 'class', 'style')
    NITF_CELL_ALIGN = ('align', 'char', 'charoff')
    NITF_CELL_VALIGN = ('valign',)
    NITF_ALLOWED_ATTR = {
        'p': NITF_COMMON_ATTR + ('lede', 'summary', 'optional-tex'),
        'a': NITF_COMMON_ATTR + ('href', 'name', 'rel', 'rev', 'title'),
        'br': ('id',),
        'em': NITF_COMMON_ATTR,
        'q': NITF_COMMON_ATTR,
        'hl1': NITF_COMMON_ATTR,
        'hl2': NITF_COMMON_ATTR,
        'table': NITF_COMMON_ATTR + (
            'tabletype',
            'align',
            'width',
            'cols',
            'border',
            'frame',
            'rules',
            'cellspacing',
            'cellpadding'),
        'tbody': NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN,
        'tr': NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN,
        'th': NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN + (
            'axis',
            'axes',
            'nowrap',
            'rowspan',
            'colspan'),
        'td': NITF_COMMON_ATTR + NITF_CELL_ALIGN + NITF_CELL_VALIGN + (
            'axis',
            'axes',
            'nowrap',
            'rowspan',
            'colspan'),
        'nitf-table': ('id',),
        'ol': NITF_COMMON_ATTR + ('seqnum',),
        'ul': NITF_COMMON_ATTR,
        'li': NITF_COMMON_ATTR,
        'dl': NITF_COMMON_ATTR,
        'dt': NITF_COMMON_ATTR,
        'dd': NITF_COMMON_ATTR,
        'bq': NITF_COMMON_ATTR + ('nowrap', 'quote-source'),
        'fn': NITF_COMMON_ATTR,
        'note': NITF_COMMON_ATTR + ('noteclass', 'type'),
        'pre': NITF_COMMON_ATTR,
        'sup': NITF_COMMON_ATTR,
        'sub': NITF_COMMON_ATTR,
        'hr': NITF_COMMON_ATTR,
    }

    NITF_ENRICHED_TEXT_PARENTS = ("a", "caption", "credit", "dd", "dt", "em", "hl1", "hl2", "lang", "li",
                                  "media-caption", "media-producer", "p", "pronounce", "q", "tagline", "td", "th")

    def __init__(self):
        Formatter.__init__(self)

        # in elements' dicts, following key can be used:
        # - 'nitf': new NITF compatible element to use (empty string to remove)
        # - 'attrib': new attribute to use (replace existing one if set)
        # - 'filter': callback to use for complex changes (root_elment and element as arguments)
        self.HTML2NITF = {
            'p': {},
            'b': {
                'nitf': 'em',
                'attrib': {'class': 'bold'}},
            'br': {'filter': self.br_filter},
            'strong': {
                'nitf': 'em',
                'attrib': {'class': 'bold'}},
            'i': {
                'nitf': 'em',
                'attrib': {'class': 'italic'}},
            'em': {
                'nitf': 'em',
                'attrib': {'class': 'italic'}},
            'u': {
                'nitf': 'em',
                'attrib': {'class': 'underscore'}},
            'strike': {'nitf': 'em'},
            'sup': {},
            'sub': {},
            'a': {},
            'img': {'nitf': ''},  # <img> use <media> in nitf, so we remove element
            'blockquote': {'nitf': 'bq'},
            'pre': {},
            'ol': {},
            'ul': {},
            'li': {},
            # FIXME: hl1 is not used here as it can only appear in <hedline>
            'h1': {'nitf': 'hl2'},
            'h2': {'nitf': 'hl2'},
            'h3': {'nitf': 'hl2'},
            'h4': {'nitf': 'hl2'},
            'h5': {'nitf': 'hl2'},
            'h6': {'nitf': 'hl2'},
            # tables
            'table': {},
            'tbody': {},
            'tr': {},
            'td': {},
            'th': {},
            'style': {'nitf': EraseElement},  # <style> may be there in case of bad paste
        }

    def format(self, article, subscriber, codes=None):
        try:
            pub_seq_num = superdesk.get_resource_service('subscribers').generate_sequence_number(subscriber)

            nitf = self.get_nitf(article, subscriber, pub_seq_num)
            return [(pub_seq_num, self.XML_ROOT + etree.tostring(nitf, pretty_print=True).decode('utf-8'))]
        except Exception as ex:
            raise FormatterError.nitfFormatterError(ex, subscriber)

    def get_nitf(self, article, destination, pub_seq_num):
        if app.config.get('NITF_INCLUDE_SCHEMA', False):
            self._message_attrib.update(self._debug_message_extra)
        nitf = etree.Element("nitf", attrib=self._message_attrib)
        head = SubElement(nitf, "head")
        body = SubElement(nitf, "body")
        body_head = SubElement(body, "body.head")
        body_content = SubElement(body, "body.content")

        body_end = SubElement(body, "body.end")

        self._format_title(article, head)
        self._format_meta(article, head, destination, pub_seq_num)
        self._format_head(article, head)
        self._format_body_head(article, body_head)
        self._format_body_content(article, body_content)
        self._format_body_end(article, body_end)
        return nitf

    def _textToParent(self, parent, children, idx, text):
        """Copy Element.text to parent or sibling element

        for internal use only
        """
        # we copy text to sibling element's tail
        # or to parent text if it is the first element
        if idx > 0:
            sibling = children[idx - 1]
            sibling.tail = (sibling.tail or '') + text
        else:
            parent.text = (parent.text or '') + text

    def _parse_children(self, html_elem, root_elem, attr_remove):
        children = list(html_elem)
        idx = 0
        while idx < len(children):
            child = children[idx]
            try:
                self.html2nitf(child, root_elem=root_elem, attr_remove=attr_remove)
            except ValueError:
                # the element is unknown
                # we need to save its text and tail,
                # and put its children to parent
                grandchildren = list(child)

                if child.text:
                    self._textToParent(html_elem, children, idx, child.text)

                if child.tail:
                    # we copy tail to last grandchild tail
                    # or we append to parent/sibling if there is no grandchild
                    if grandchildren:
                        last = grandchildren[-1]
                        last.tail = (last.tail or '') + child.tail
                    else:
                        self._textToParent(html_elem, children, idx, child.tail)

                # we move elem children to parent
                for grandchild_idx, grandchild in enumerate(grandchildren):
                    insert_idx = idx + grandchild_idx
                    html_elem.insert(insert_idx, grandchild)
                    children.insert(insert_idx, grandchild)

                # and remove the element
                html_elem.remove(child)
                children.remove(child)

                # and we continue with the same index, so new children are parsed
                continue
            except EraseElement:
                # the element need to be fully erased (itself and its children)
                html_elem.remove(child)
                children.remove(child)
                continue
            idx += 1

        return html_elem

    def html2nitf(self, html_elem, root_elem=None, attr_remove=None):
        """Convert HTML elements to NITF compatible elements

        :param ET.Element: HTML to clean/transform
        :param ET.Element: None if its the root element (must be a <div>)
        :param list attr_remove: attributes to remove if present
        :return ET.Element: <div> element with NITF compliant children
        """
        if root_elem is None:
            root_elem = html_elem
            assert html_elem.tag == 'div'
            # we change children of root element in place
            return self._parse_children(html_elem, root_elem, attr_remove)

        try:
            nitf_map = self.HTML2NITF[html_elem.tag]
        except KeyError:
            raise ValueError("Unhandled HTML element")
        nitf_elem = nitf_map.get('nitf')
        if nitf_elem is not None:
            if nitf_elem == '':
                raise ValueError("Element need to be removed")
            elif nitf_elem == EraseElement:
                raise EraseElement("Element need to be erased")
            html_elem.tag = nitf_elem

        if attr_remove is not None:
            for attr in attr_remove:
                if attr in html_elem.attrib:
                    del html_elem.attrib[attr]

        html_elem.attrib.update(nitf_map.get('attrib', {}))

        if 'filter' in nitf_map:
            nitf_map['filter'](root_elem, html_elem)

        attr_allowed = self.NITF_ALLOWED_ATTR.get(html_elem.tag, ())

        for attr in list(html_elem.attrib):
            if attr not in attr_allowed:
                del html_elem.attrib[attr]

        return self._parse_children(html_elem, root_elem, attr_remove)

    def br_filter(self, root_elem, html_elem):
        parent = html_elem.find('..')
        if parent.tag not in self.NITF_ENRICHED_TEXT_PARENTS:
            raise ValueError(u"this element need to be removed")
        elif parent.tag == 'p':
            if parent[-1] == html_elem and not (html_elem.tail or '').strip():
                raise ValueError(u"this element need to be removed")

    def _format_tobject(self, article, head):
        return SubElement(head, 'tobject', {'tobject.type': 'news'})

    def _format_docdata_dateissue(self, article, docdata):
        SubElement(docdata, 'date.issue', {'norm': str(article.get('firstcreated', ''))})

    def _format_docdata_doc_id(self, article, docdata):
        SubElement(docdata, 'doc-id', attrib={'id-string': article.get('guid', '')})

    def _format_docdata(self, article, docdata):
        SubElement(docdata, 'urgency', {'ed-urg': str(article.get('urgency', ''))})
        self._format_docdata_dateissue(article, docdata)
        self._format_docdata_doc_id(article, docdata)

        # ednote can exist and be set to None, that why we can't use default
        # value for article.get
        SubElement(docdata, 'ed-msg', {'info': article.get('ednote') or ''})

    def _format_pubdata(self, article, head):
        pass

    def _format_title(self, article, head):
        title = SubElement(head, 'title')
        title.text = article.get('headline', '')

    def _format_date_expire(self, article, docdata):
        if article.get(EMBARGO):
            docdata.attrib['management-status'] = 'embargoed'
            SubElement(docdata, 'date.expire',
                       {'norm': str(get_utc_schedule(article, EMBARGO).isoformat())})
        else:
            docdata.attrib['management-status'] = article.get('pubstatus', '')
            SubElement(docdata, 'date.expire', {'norm': str(article.get('expiry', ''))})

    def _format_head(self, article, head):
        tobject = self._format_tobject(article, head)
        if 'genre' in article and len(article['genre']) > 0:
            SubElement(tobject, 'tobject.property', {'tobject.property.type': article['genre'][0]['name']})
        self._format_subjects(article, tobject)

        docdata = SubElement(head, 'docdata')
        self._format_date_expire(article, docdata)
        self._format_docdata(article, docdata)
        self._format_pubdata(article, head)
        self._format_keywords(article, head)

    def _format_subjects(self, article, tobject):
        for subject in article.get('subject', []):
            SubElement(tobject, 'tobject.subject',
                       {'tobject.subject.refnum': subject.get('qcode', '')})

    def _format_keywords(self, article, head):
        if article.get('keywords'):
            keylist = SubElement(head, 'key-list')
            for keyword in article['keywords']:
                SubElement(keylist, 'keyword', {'key': keyword})

    def _format_body_head_abstract(self, article, body_head):
        if article.get('abstract'):
            abstract = SubElement(body_head, 'abstract')
            self.map_html_to_xml(abstract, article.get('abstract'))

    def _format_body_head_distributor(self, article, body_head):
        pass

    def _format_body_head_dateline(self, article, body_head):
        if article.get('dateline', {}).get('text'):
            dateline = SubElement(body_head, 'dateline')
            dateline.text = article['dateline']['text']

    def _format_body_head(self, article, body_head):
        hedline = SubElement(body_head, 'hedline')
        hl1 = SubElement(hedline, 'hl1')
        hl1.text = article.get('headline', '')

        if article.get('byline'):
            byline = SubElement(body_head, 'byline')
            byline.text = article['byline']

        self._format_body_head_distributor(article, body_head)
        self._format_body_head_dateline(article, body_head)
        self._format_body_head_abstract(article, body_head)

        for company in article.get('company_codes', []):
            org = SubElement(body_head, 'org', attrib={'idsrc': company.get('security_exchange', ''),
                                                       'value': company.get('qcode', '')})
            org.text = company.get('name', '')

    def _format_body_content(self, article, body_content):
        if article.get(FORMAT) == FORMATS.PRESERVED:
            pre = get_text(self.append_body_footer(article))
            SubElement(body_content, 'pre').text = pre
        else:
            self.map_html_to_xml(body_content, self.append_body_footer(article))

    def _format_body_end(self, article, body_end):
        if article.get('ednote'):
            tagline = SubElement(body_end, 'tagline')
            tagline.text = article['ednote']

    def can_format(self, format_type, article):
        return format_type == 'nitf' and \
            article[ITEM_TYPE] in (CONTENT_TYPE.TEXT, CONTENT_TYPE.PREFORMATTED, CONTENT_TYPE.COMPOSITE)

    def _format_meta_priority(self, article, head):
        pass

    def _format_meta(self, article, head, destination, pub_seq_num):
        """
        Appends <meta> elements to <head>
        """
        self._format_meta_priority(article, head)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging

from superdesk import get_resource_service
from superdesk.notification import push_notification
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.utils import SuperdeskBaseEnum
from flask import current_app as app

logger = logging.getLogger(__name__)


PUBLISHED_IN_PACKAGE = 'published_in_package'


class QueueState(SuperdeskBaseEnum):
    PENDING = 'pending'
    IN_PROGRESS = 'in-progress'
    RETRYING = 'retrying'
    SUCCESS = 'success'
    CANCELED = 'canceled'
    ERROR = 'error'
    FAILED = 'failed'


class PublishQueueResource(Resource):
    schema = {
        'item_id': Resource.rel('archive', type='string'),
        'item_version': {'type': 'integer', 'nullable': False},

        'formatted_item': {'type': 'string', 'nullable': False},
        'item_encoding': {'type': 'string', 'nullable': True},
        'encoded_item_id': {'type': 'objectid', 'nullable': True},
        'subscriber_id': Resource.rel('subscribers'),
        'codes': {'type': 'list', 'nullable': True},
        'destination': {
            'type': 'dict',
            'schema': {
                'name': {'type': 'string', 'required': True, 'empty': False},
                'format': {'type': 'string', 'required': True},
                'delivery_type': {'type': 'string', 'required': True},
                'config': {'type': 'dict'}
            }
        },
        PUBLISHED_IN_PACKAGE: {
            'type': 'string'
        },
        'published_seq_num': {
            'type': 'integer'
        },
        # publish_schedule is to indicate the item schedule datetime.
        # entries in the queue are created after schedule has elapsed.
        'publish_schedule': {
            'type': 'datetime'
        },
        'publishing_action': {
            'type': 'string'
        },
        'unique_name': {
            'type': 'string'
        },
        'content_type': {
            'type': 'string'
        },
        'headline': {
            'type': 'string'
        },

        'transmit_started_at': {
            'type': 'datetime'
        },
        'completed_at': {
            'type': 'datetime'
        },
        'state': {
            'type': 'string',
            'allowed': QueueState.values(),
            'nullable': False
        },
        'error_message': {
            'type': 'string'
        },
        # to indicate the queue item is moved to legal
        # True is set after state of the item is success, cancelled or failed. For other state it is false
        'moved_to_legal': {
            'type': 'boolean',
            'default': False
        },
        'retry_attempt': {
            'type': 'integer',
            'default': 0
        },
        'next_retry_attempt_at': {
            'type': 'datetime'
        },
        'ingest_provider': Resource.rel('ingest_providers', nullable=True),
        'associated_items': {'type': 'list', 'nullable': True}
    }

    additional_lookup = {
        'url': r'regex("[\w,.:-]+")',
        'field': 'item_id'
    }

    etag_ignore_fields = ['moved_to_legal']
    datasource = {'default_sort': [('_created', -1), ('subscriber_id', 1), ('published_seq_num', -1)]}
    privileges = {'POST': 'publish_queue', 'PATCH': 'publish_queue'}


class PublishQueueService(BaseService):

    def on_create(self, docs):
        subscriber_service = get_resource_service('subscribers')

        for doc in docs:
            self._set_queue_state(doc, {})
            doc['moved_to_legal'] = False

            if 'published_seq_num' not in doc:
                subscriber = subscriber_service.find_one(req=None, _id=doc['subscriber_id'])
                doc['published_seq_num'] = subscriber_service.generate_sequence_number(subscriber)

    def on_updated(self, updates, original):
        if updates.get('state', '') != original.get('state', ''):
            self._set_queue_state(updates, original)
            push_notification('publish_queue:update',
                              queue_id=str(original['_id']),
                              completed_at=(updates.get('completed_at').isoformat()
                                            if updates.get('completed_at') else None),
                              state=updates.get('state'),
                              error_message=updates.get('error_message')
                              )

    def _set_queue_state(self, updates, original):
        destination = updates.get('destination', original.get('destination')) or {}
        updates['state'] = QueueState.SUCCESS.value if destination.get('delivery_type') == 'content_api' \
            else updates.get('state') or QueueState.PENDING.value

    def delete(self, lookup):
        # as encoded item is added manually to storage
        # we also need to remove it manually on delete
        cur = self.get_from_mongo(req=None, lookup=lookup)
        for doc in cur:
            try:
                encoded_item_id = doc['encoded_item_id']
            except KeyError:
                pass
            else:
                app.storage.delete(encoded_item_id)
        return super().delete(lookup)

    def delete_by_article_id(self, _id):
        lookup = {'item_id': _id}
        self.delete(lookup=lookup)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase
from superdesk.io.feed_parsers.ninjs import NINJSFeedParser


class NINJSTestCase(TestCase):
    vocab = [{'_id': 'genre', 'items': [{'name': 'Current'}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert('vocabularies', self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        self.items = NINJSFeedParser().parse(fixture, provider)


class SimpleTestCase(NINJSTestCase):

    filename = 'ninjs1.json'

    def test_headline(self):
        self.assertEqual(self.items[0].get('headline'), "headline")
        self.assertEqual(self.items[0].get('description_text'), "abstract")
        self.assertEqual(1, len(self.items[0].get('authors')), 'authors')
        self.assertEqual(self.items[0]['authors'][0], {
            'name': 'John',
            'role': 'writer',
            'avatar_url': 'http://example.com',
            'biography': 'bio',
        })


class AssociatedTestCase(NINJSTestCase):

    filename = 'ninjs2.json'

    def test_parsed_items(self):
        # The picture
        self.assertEqual(self.items[0].get('type'), 'picture')
        self.assertEqual(self.items[0].get('headline'), 'Financial Markets')
        self.assertEqual(self.items[0].get('alt_text'), 'Oil markets something')
        self.assertEqual(self.items[0].get('description_text'),
                         'Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.')
        # The text item
        self.assertEqual(self.items[1].get('type'), 'text')
        self.assertEqual(self.items[1].get('headline'), 'Oil prices edge up on first drop in US drilling in months')
        self.assertEqual(self.items[1].get('abstract'),
                         'Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.')
        # The associated picture in the text item
        self.assertEqual(self.items[1].get('associations').get('featuremedia').get('type'), 'picture')
        self.assertEqual(self.items[1].get('associations').get('featuremedia').get('alt_text'), 'Oil markets something')
        self.assertEqual(self.items[1].get('associations').get('featuremedia').get('description_text'),
                         'Oil prices edged up on Monday, supported by the first fall in US drilling activity in month.')


class PictureTestCase(NINJSTestCase):

    filename = 'ninjs3.json'

    def test_headline(self):
        self.assertEqual(self.items[0].get('headline'), 'German Air Force Museum')
        self.assertEqual(self.items[0].get('type'), "picture")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
from superdesk.tests import TestCase

from superdesk.etree import etree
from superdesk.io.feed_parsers import wordpress_wxr
from unittest import mock
import datetime

FAKE_MEDIA_DATA = {'_created': datetime.datetime(2017, 4, 26, 13, 0, 33, tzinfo=datetime.timezone.utc),
                   '_etag': '0b135dc3959fe4b09642d02ae733b94af72af5d3',
                   '_id': '590099f1cc3a2d2349a785f4',
                   '_updated': datetime.datetime(2017, 4, 26, 13, 0, 33, tzinfo=datetime.timezone.utc),
                   'media': '590099f1cc3a2d2349a785ee',
                   'mimetype': 'image/jpeg',
                   'renditions': {'original': {'height': 256,
                                               'href': 'http://test',
                                               'media': '590099f1cc3a2d2349a785ee',
                                               'mimetype': 'image/jpeg',
                                               'width': 642},
                                  'thumbnail': {'height': 23,
                                                'href': 'http://test',
                                                'media': '590099f1cc3a2d2349a785f0',
                                                'mimetype': 'image/jpeg',
                                                'width': 60},
                                  'viewImage': {'height': 79,
                                                'href': 'http://test',
                                                'media': '590099f1cc3a2d2349a785f2',
                                                'mimetype': 'image/jpeg',
                                                'width': 200}}}


def fake_update_renditions(item, url, _):
    update = {
        # we use URL as _id so we can check the value easily
        '_id': url,
        'renditions': {'original': {'height': 256,
                                    'href': 'http://test',
                                    'media': '590099f1cc3a2d2349a785ee',
                                    'mimetype': 'image/jpeg',
                                    'width': 642},
                       'thumbnail': {'height': 23,
                                     'href': 'http://test',
                                     'media': '590099f1cc3a2d2349a785f0',
                                     'mimetype': 'image/jpeg',
                                     'width': 60},
                       'viewImage': {'height': 79,
                                     'href': 'http://test',
                                     'media': '590099f1cc3a2d2349a785f2',
                                     'mimetype': 'image/jpeg',
                                     'width': 200}},
        'mimetype': 'image/jpeg'}
    item.update(update)


class WPWXRTestCase(TestCase):

    filename = 'wordpress_wxr.xml'

    @mock.patch.object(wordpress_wxr, 'update_renditions', fake_update_renditions)
    def __init__(self, methodname):
        super().__init__(methodname)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, '../fixtures', self.filename))
        provider = {'name': 'Test'}
        with open(fixture, 'rb') as f:
            buf = f.read()
        self.ori_file = buf
        buf = buf.replace(b'\r', b'&#13;')
        parser = etree.XMLParser(recover=True)
        parsed = etree.fromstring(buf, parser)
        self.articles = wordpress_wxr.WPWXRFeedParser().parse(parsed, provider)

    def test_guid(self):
        self.assertEqual(self.articles[0]['guid'], 'http://sdnewtester.org/?p=216')

    def test_versioncreated(self):
        self.assertEqual(self.articles[0]['versioncreated'],
                         datetime.datetime(2013, 7, 29, 16, 3, 54, tzinfo=datetime.timezone.utc))

    def test_author(self):
        self.assertEqual(self.articles[0]['author'], 'admin')

    def test_headline(self):
        self.assertEqual(self.articles[0]['headline'], 'Starafrica to dispose assets for $6 million to clear debt')

    def test_body_html(self):
        expected = ('<p>By Tester\nHarare, July 19 (The SDNewTester) - '
                    'Cash-strapped StarAfrica Corporation is set to dis'
                    'pose its transport company and its stake in Tongaa'
                    't Hulett Botswana for $6 million to offset part of'
                    ' the companyâs $19.7 million debt.\nBla bla test</p>')
        self.assertEqual(self.articles[0]['body_html'], expected)

    def test_keywords(self):
        self.assertEqual(self.articles[0]['keywords'], ['companies'])

    def test_category(self):
        self.assertEqual(self.articles[0]['anpa_category'],
                         [{'qcode': 'Business', 'qname': 'Business'},
                          {'qcode': 'Companies', 'qname': 'Companies'},
                          {'qcode': 'Economy', 'qname': 'Economy'}])

    def test_attachments(self):
        # in self.articles[1] there is a body, so image should be in associations
        expected = {'_id': 'http://example.net/image.png',
                    'ingest_provider': 'wpwxr',
                    'headline': 'test2',
                    'alt_text': ' ',
                    'description_text': ' ',
                    'mimetype': 'image/jpeg',
                    'renditions': {'original': {'height': 256,
                                                'href': 'http://test',
                                                'media': '590099f1cc3a2d2349a785ee',
                                                'mimetype': 'image/jpeg',
                                                'width': 642},
                                   'thumbnail': {'height': 23,
                                                 'href': 'http://test',
                                                 'media': '590099f1cc3a2d2349a785f0',
                                                 'mimetype': 'image/jpeg',
                                                 'width': 60},
                                   'viewImage': {'height': 79,
                                                 'href': 'http://test',
                                                 'media': '590099f1cc3a2d2349a785f2',
                                                 'mimetype': 'image/jpeg',
                                                 'width': 200}},
                    'type': 'picture'}
        self.assertEqual(self.articles[1]['associations']['featuremedia'], expected)

    def test_attachments_no_body(self):
        # in self.articles[1] there is no body, so item should be an image
        # cf. SDTS-29
        expected = {'original': {'height': 256,
                                 'href': 'http://test',
                                 'media': '590099f1cc3a2d2349a785ee',
                                 'mimetype': 'image/jpeg',
                                 'width': 642},
                    'thumbnail': {'height': 23,
                                  'href': 'http://test',
                                  'media': '590099f1cc3a2d2349a785f0',
                                  'mimetype': 'image/jpeg',
                                  'width': 60},
                    'viewImage': {'height': 79,
                                  'href': 'http://test',
                                  'media': '590099f1cc3a2d2349a785f2',
                                  'mimetype': 'image/jpeg',
                                  'width': 200}}
        self.assertNotIn('associations', self.articles[3])
        self.assertEqual(self.articles[3]['renditions'], expected)
        self.assertEqual(self.articles[3]['type'], 'picture')

    def test_clrf(self):
        expected = ('<p>By Tester</p><p>Harare, July 19 (The SDNewTester) - Cash-strapped'
                    ' StarAfrica Corporation is set to dispose its transport company and '
                    'its stake in Tongaat Hulett Botswana for $6 million to offset part o'
                    'f the companyâs $19.7 million debt.</p><hr><p>Bla bla test</p>')
        self.assertEqual(self.articles[2]['body_html'], expected)




import superdesk

from flask import current_app as app
from apps.auth import is_current_user_admin


class ConfigResource(superdesk.Resource):
    schema = {
        '_id': {'type': 'string', 'required': True},
        'val': {'type': 'dict'},
    }

    item_url = 'regex("[\w.:_-]+")'
    item_methods = ['GET']
    resource_methods = ['POST']


class ConfigService(superdesk.Service):

    def find_one(self, req, **lookup):
        item = super().find_one(req, **lookup)
        if item:
            return item
        else:
            return {'_id': lookup.get('_id'), 'val': None}

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            self.set(doc['_id'], doc.get('val'))
            ids.append(doc['_id'])
        return ids

    def set(self, key, val, namespace='superdesk'):
        coll = app.data.mongo.get_collection_with_write_concern('config', 'config')
        coll.update_one({'_id': key}, {'$set': {'_id': key, 'val': val}}, upsert=True)

    def get(self, key, namespace='superdesk'):
        return self.find_one(req=None, _id=key).get('val')

    def is_authorizes(self, user):
        return is_current_user_admin()


def init_app(app):
    superdesk.register_resource('config', ConfigResource, ConfigService, _app=app)
    superdesk.intrinsic_privilege('config', method=['POST'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from io import BytesIO
from unittest import mock

from superdesk.tests import TestCase
from superdesk import get_resource_service
from ..media import get_picture_fixture

from apps.picture_crop import get_crop_size


def get_file_mock(rendition, item):
    filename = get_picture_fixture()
    with open(filename, 'rb') as imgfile:
        data = BytesIO(imgfile.read())
    return data


def crop_image_mock(content, file_name, cropping_data, exact_size=None, image_format=None):
    setattr(content, 'width', 1)
    setattr(content, 'height', 1)
    return True, content


def media_put_mock(content, filename=None, content_type=None, metadata=None, resource=None, **kwargs):
    media = mock.MagicMock()
    media.name = 'test.jpg'
    media.metadata = metadata
    return media


class PictureCropServiceTest(TestCase):
    @mock.patch('apps.picture_crop.get_file', side_effect=get_file_mock)
    @mock.patch('apps.picture_crop.crop_image', side_effect=crop_image_mock)
    @mock.patch('superdesk.app.media.put', side_effect=media_put_mock)
    def test_crop_image_copies_metadata(self, get_file_function, crop_image_function, media_put_function):
        service = get_resource_service('picture_crop')

        images = service.post([{
            'item': {
                'renditions': {
                    'original': {
                        'mimetype': 'image/jpeg'
                    }
                }
            },
            'crop': {
                'CropLeft': 10,
                'CropRight': 100,
                'CropTop': 10,
                'CropBottom': 100
            }
        }])

        self.assertEqual(images[0].metadata.get('datetime'), '"2015:07:06 16:30:23"')
        self.assertEqual(images[0].metadata.get('exifimagewidth'), '400')
        self.assertEqual(images[0].metadata.get('exifimageheight'), '300')

    def test_get_crop_size_fixes_crop_aspect_ratio(self):
        crop_data = {
            'CropLeft': 0,
            'CropRight': 1620,
            'CropTop': 0,
            'CropBottom': 1230,
            'width': 800,
            'height': 600
        }
        get_crop_size(crop_data)

        crop_width = crop_data['CropRight'] - crop_data['CropLeft']
        crop_height = crop_data['CropBottom'] - crop_data['CropTop']

        crop_ratio = crop_height / crop_width
        size_ratio = crop_data['height'] / crop_data['width']

        self.assertEqual(round(crop_ratio, 3), round(size_ratio, 3))

    def test_get_crop_size_uses_full_picture(self):
        crop_data = {
            'width': 1290,
            'height': 490,
            'CropTop': 59,
            'CropRight': 1300,
            'CropBottom': 554,
            'CropLeft': 0,
        }

        crop_size = get_crop_size(crop_data)
        self.assertEqual(1290, crop_size['width'])
        self.assertEqual(490, crop_size['height'])
        self.assertEqual(0, crop_data['CropLeft'])
        self.assertEqual(1300, crop_data['CropRight'])
        self.assertEqual(59, crop_data['CropTop'])
        self.assertGreaterEqual(554, crop_data['CropBottom'])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import json
from flask import current_app as app
from superdesk import get_resource_service
from superdesk.services import BaseService
from eve.utils import ParsedRequest
from superdesk.notification import push_notification
from apps.archive.common import get_user
from eve.utils import config
from superdesk.utc import utcnow
from apps.archive.common import ITEM_MARK, ITEM_UNMARK


def get_marked_items(desk_id):
    """Get items marked for given desk"""
    query = {
        'query': {'filtered': {'filter': {'term': {'marked_desks.desk_id': str(desk_id)}}}},
        'sort': [{'versioncreated': 'desc'}],
        'size': 200
    }
    request = ParsedRequest()
    request.args = {'source': json.dumps(query), 'repo': 'archive,published'}
    return list(get_resource_service('search').get(req=request, lookup=None))


class MarkedForDesksService(BaseService):
    def create(self, docs, **kwargs):
        """Toggle marked desk status for given desk and item."""

        service = get_resource_service('archive')
        published_service = get_resource_service('published')
        ids = []
        for doc in docs:
            item = service.find_one(req=None, guid=doc['marked_item'])
            if not item:
                ids.append(None)
                continue
            ids.append(item['_id'])
            marked_desks = item.get('marked_desks', [])
            if not marked_desks:
                marked_desks = []

            existing_mark = next((m for m in marked_desks if m['desk_id'] == doc['marked_desk']), None)

            if existing_mark:
                # there is an existing mark so this is un-mark action
                marked_desks = [m for m in marked_desks if m['desk_id'] != doc['marked_desk']]
                marked_desks_on = False  # highlight toggled off
            else:
                # there is no existing mark so this is mark action
                new_mark = {}
                new_mark['desk_id'] = doc['marked_desk']
                new_mark['user_marked'] = str(get_user(True).get(config.ID_FIELD, ''))
                new_mark['date_marked'] = utcnow()
                marked_desks.append(new_mark)
                marked_desks_on = True

            updates = {'marked_desks': marked_desks}
            service.system_update(item['_id'], updates, item)

            publishedItems = published_service.find({'item_id': item['_id']})
            for publishedItem in publishedItems:
                if publishedItem['_current_version'] == item['_current_version'] or not marked_desks_on:
                    updates = {'marked_desks': marked_desks}
                    published_service.system_update(publishedItem['_id'], updates, publishedItem)

            push_notification(
                'item:marked_desks',
                marked=int(marked_desks_on),
                item_id=item['_id'],
                mark_id=str(doc['marked_desk']))

            if marked_desks_on:
                app.on_archive_item_updated({'desk_id': doc['marked_desk']}, item, ITEM_MARK)
            else:
                app.on_archive_item_updated({'desk_id': doc['marked_desk']}, item, ITEM_UNMARK)

        return ids



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import functools
import logging
import os
import socket
import unittest
from base64 import b64encode
from unittest.mock import patch

from flask import json, Config

from apps.ldap import ADAuth
from superdesk import get_resource_service
from superdesk.factory import get_app

logger = logging.getLogger(__name__)
test_user = {
    'username': 'test_user',
    'password': 'test_password',
    'is_active': True,
    'is_enabled': True,
    'needs_activation': False,
    'sign_off': 'abc',
    'email': 'behave_test@sourcefabric.org',
    'preferences': {
        'email:notification': {
            'label': 'Send notifications via email',
            'type': 'bool',
            'default': True,
            'category': 'notifications',
            'enabled': True}
    }
}


def get_mongo_uri(key, dbname):
    """Read mongo uri from env variable and replace dbname.

    :param key: env variable name
    :param dbname: mongo db name to use
    """
    env_uri = os.environ.get(key, 'mongodb://localhost/test')
    env_host = env_uri.rsplit('/', 1)[0]
    return '/'.join([env_host, dbname])


def update_config(conf):
    conf['ELASTICSEARCH_INDEX'] = 'sptest'
    conf['MONGO_URI'] = get_mongo_uri('MONGO_URI', 'sptests')
    conf['LEGAL_ARCHIVE_DBNAME'] = 'sptests_legal_archive'
    conf['LEGAL_ARCHIVE_URI'] = get_mongo_uri('LEGAL_ARCHIVE_URI', 'sptests_legal_archive')
    conf['ARCHIVED_DBNAME'] = 'sptests_archived'
    conf['ARCHIVED_URI'] = get_mongo_uri('ARCHIVED_URI', 'sptests_archived')
    conf['CONTENTAPI_URL'] = 'http://localhost:5400'
    conf['CONTENTAPI_MONGO_DBNAME'] = 'sptests_contentapi'
    conf['CONTENTAPI_MONGO_URI'] = get_mongo_uri('CONTENTAPI_MONGO_URI', 'sptests_contentapi')
    conf['CONTENTAPI_ELASTICSEARCH_INDEX'] = 'sptest_contentapi'

    conf['DEBUG'] = True
    conf['TESTING'] = True
    conf['SUPERDESK_TESTING'] = True
    conf['BCRYPT_GENSALT_WORK_FACTOR'] = 4
    conf['CELERY_TASK_ALWAYS_EAGER'] = 'True'
    conf['CELERY_BEAT_SCHEDULE_FILENAME'] = './testschedule.db'
    conf['CELERY_BEAT_SCHEDULE'] = {}
    conf['CONTENT_EXPIRY_MINUTES'] = 99
    conf['VERSION'] = '_current_version'
    conf['SECRET_KEY'] = 'test-secret'
    conf['JSON_SORT_KEYS'] = True
    conf['ELASTICSEARCH_INDEXES'] = {
        'archived': 'sptest_archived',
        'archive': 'sptest_archive',
        'ingest': 'sptest_ingest',
    }

    # (behave|nose)tests depends from these settings
    conf['DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES'] = 'AAP'
    conf['MACROS_MODULE'] = 'superdesk.macros'
    conf['DEFAULT_TIMEZONE'] = 'Europe/Prague'
    conf['LEGAL_ARCHIVE'] = True

    # limit mongodb connections
    conf['MONGO_CONNECT'] = False
    conf['ARCHIVED_CONNECT'] = False
    conf['LEGAL_ARCHIVE_CONNECT'] = False
    conf['MONGO_MAX_POOL_SIZE'] = 1
    conf['ARCHIVED_MAX_POOL_SIZE'] = 1
    conf['LEGAL_ARCHIVE_MAX_POOL_SIZE'] = 1
    conf['PUBLISH_ASSOCIATED_ITEMS'] = True
    return conf


def drop_elastic(app):
    with app.app_context():
        es = app.data.elastic.es
        indexes = [app.config['ELASTICSEARCH_INDEX']] + list(app.config['ELASTICSEARCH_INDEXES'].values())
        for index in indexes:
            es.indices.delete(index, ignore=[404])


def foreach_mongo(fn):
    """
    Run the same actions on all mongo databases

    This decorator adds two additional parameters to called function
    `dbconn` and `dbname` for using proper connection and database name
    """
    @functools.wraps(fn)
    def inner(app, *a, **kw):
        pairs = (
            ('MONGO', 'MONGO_DBNAME'),
            ('ARCHIVED', 'ARCHIVED_DBNAME'),
            ('LEGAL_ARCHIVE', 'LEGAL_ARCHIVE_DBNAME'),
            ('CONTENTAPI_MONGO', 'CONTENTAPI_MONGO_DBNAME')
        )
        with app.app_context():
            for prefix, name in pairs:
                if not app.config.get(name):
                    continue
                kw['dbname'] = app.config[name]
                kw['dbconn'] = app.data.mongo.pymongo(prefix=prefix).cx
                fn(app, *a, **kw)
    return inner


@foreach_mongo
def drop_mongo(app, dbconn, dbname):
    dbconn.drop_database(dbname)
    dbconn.close()


def setup_config(config):
    app_abspath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    app_config = Config(app_abspath)
    app_config.from_object('superdesk.default_settings')

    update_config(app_config)
    app_config.update(config or {}, **{
        'APP_ABSPATH': app_abspath,
        'DEBUG': True,
        'TESTING': True,
    })

    logging.getLogger('apps').setLevel(logging.WARNING)
    logging.getLogger('elastic').setLevel(logging.WARNING)  # elastic datalayer
    logging.getLogger('urllib3').setLevel(logging.WARNING)
    logging.getLogger('celery').setLevel(logging.WARNING)
    logging.getLogger('superdesk').setLevel(logging.ERROR)
    logging.getLogger('elasticsearch').setLevel(logging.ERROR)
    logging.getLogger('superdesk.errors').setLevel(logging.CRITICAL)
    return app_config


def clean_dbs(app, force=False):
    clean_es(app, force)
    drop_mongo(app)


def retry(exc, count=1):
    def wrapper(fn):
        num = 0

        @functools.wraps(fn)
        def inner(*a, **kw):
            global num

            try:
                return fn(*a, **kw)
            except exc as e:
                logging.exception(e)
                if num < count:
                    num += 1
                    return inner(*a, **kw)
        return inner
    return wrapper


def _clean_es(app):
    indices = '%s*' % app.config['ELASTICSEARCH_INDEX']
    es = app.data.elastic.es
    es.indices.delete(indices, ignore=[404])
    with app.app_context():
        app.data.init_elastic(app)


@retry(socket.timeout, 2)
def clean_es(app, force=False):
    use_snapshot(app, 'clean', [snapshot_es], force)(_clean_es)(app)


def snapshot(fn):
    """
    Call create or restore snapshot function
    """
    @functools.wraps(fn)
    def inner(app, name, action, **kw):
        assert action in ['create', 'restore']
        create, restore = fn(app, name, **kw)
        {'create': create, 'restore': restore}[action]()
    return inner


@snapshot
def snapshot_es(app, name):
    indices = '%s*' % app.config['ELASTICSEARCH_INDEX']
    backup = ('backups', '%s%s' % (indices[:-1], name))
    es = app.data.elastic.es

    def create():
        es.snapshot.delete(*backup, ignore=[404])
        es.indices.open(indices, expand_wildcards='closed', ignore=[404])
        es.snapshot.create(*backup, wait_for_completion=True, body={
            'indices': indices,
            'allow_no_indices': False,
        })

    def restore():
        es.indices.close(indices, expand_wildcards='open', ignore=[404])
        es.snapshot.restore(*backup, body={
            'indices': indices,
            'allow_no_indices': False
        }, wait_for_completion=True)
    return create, restore


@foreach_mongo
@snapshot
def snapshot_mongo(app, name, dbconn, dbname):
    snapshot = '%s_%s' % (dbname, name)

    def create():
        dbconn.drop_database(snapshot)
        dbconn.admin.command('copydb', fromdb=dbname, todb=snapshot)

    def restore():
        dbconn.drop_database(dbname)
        dbconn.admin.command('copydb', fromdb=snapshot, todb=dbname)
    return create, restore


def use_snapshot(app, name, funcs=(snapshot_es, snapshot_mongo), force=False):
    def snapshot(action):
        for f in funcs:
            f(app, name, action=action)

    def wrapper(fn):
        path = app.config.get('ELASTICSEARCH_BACKUPS_PATH')
        enabled = path and os.path.exists(path)

        @functools.wraps(fn)
        def inner(*a, **kw):
            if not enabled or force:
                logger.debug(
                    'Don\'t use snapshot for %s; enabled=%s; force=%s',
                    fn, enabled, force
                )
                use_snapshot.cache.pop(fn, None)
                return fn(*a, **kw)

            if fn in use_snapshot.cache:
                snapshot('restore')
                logger.debug('Restore snapshot for %s', fn)
            else:
                use_snapshot.cache[fn] = fn(*a, **kw)
                snapshot('create')
                logger.debug('Create snapshot for %s', fn)
            return use_snapshot.cache[fn]
        return inner
    return wrapper


use_snapshot.cache = {}


def setup(context=None, config=None, app_factory=get_app, reset=False):
    if not hasattr(setup, 'app') or setup.reset or config:
        cfg = setup_config(config)
        setup.app = app_factory(cfg)
        setup.reset = reset
    app = setup.app

    if context:
        context.app = app
        context.client = app.test_client()

    clean_dbs(app, force=bool(config))


def setup_auth_user(context, user=None):
    setup_db_user(context, user)


def add_to_context(context, token, user, auth_id=None):
    context.headers.append(('Authorization', b'basic ' + b64encode(token + b':')))
    if getattr(context, 'user', None):
        context.previous_user = context.user
    context.user = user
    set_placeholder(context, 'CONTEXT_USER_ID', str(user.get('_id')))
    set_placeholder(context, 'AUTH_ID', str(auth_id))


def set_placeholder(context, name, value):
    old_p = getattr(context, 'placeholders', None)
    if not old_p:
        context.placeholders = dict()
    context.placeholders[name] = value


def get_prefixed_url(current_app, endpoint):
    if endpoint.startswith('http://'):
        return endpoint

    endpoint = endpoint.lstrip('/')
    url_prefix = current_app.config['URL_PREFIX'] + '/'
    if endpoint.startswith(url_prefix):
        return endpoint
    return url_prefix + endpoint


def setup_db_user(context, user):
    """Setup the user for the DB authentication.

    :param context: test context
    :param dict user: user
    """
    user = user or test_user
    with context.app.test_request_context(context.app.config['URL_PREFIX']):
        original_password = user['password']

        user.setdefault('user_type', 'administrator')

        if not get_resource_service('users').find_one(username=user['username'], req=None):
            get_resource_service('users').post([user])

        user['password'] = original_password
        auth_data = json.dumps({'username': user['username'], 'password': user['password']})
        auth_response = context.client.post(get_prefixed_url(context.app, '/auth_db'),
                                            data=auth_data, headers=context.headers)

        auth_data = json.loads(auth_response.get_data())
        token = auth_data.get('token').encode('ascii')
        auth_id = auth_data.get('_id')
        add_to_context(context, token, user, auth_id)


def setup_ad_user(context, user):
    """Setup the AD user for the LDAP authentication.

    The method patches the authenticate_and_fetch_profile method of the ADAuth class

    :param context: test context
    :param dict user: user
    """
    ad_user = user or test_user

    # This is necessary as test_user is in Global scope and del doc['password']
    # removes the key from test_user and for the next scenario,
    # auth_data = json.dumps({'username': ad_user['username'], 'password': ad_user['password']})
    # will fail as password key is removed by del doc['password']
    ad_user = ad_user.copy()
    ad_user['email'] = 'mock@mail.com.au'

    ad_user.setdefault('user_type', 'administrator')

    # ad profile to be return from the patch object
    ad_profile = {
        'email': ad_user['email'],
        'username': ad_user['username'],
        # so that test run under the administrator context.
        'user_type': ad_user.get('user_type'),
        'sign_off': ad_user.get('sign_off', 'abc'),
        'preferences': {
            'email:notification': {
                'label': 'Send notifications via email',
                'type': 'bool',
                'default': True,
                'category': 'notifications',
                'enabled': True}
        }
    }

    with patch.object(ADAuth, 'authenticate_and_fetch_profile', return_value=ad_profile):
        auth_data = json.dumps({'username': ad_user['username'], 'password': ad_user['password']})
        auth_response = context.client.post(get_prefixed_url(context.app, '/auth_db'),
                                            data=auth_data, headers=context.headers)
        auth_response_as_json = json.loads(auth_response.get_data())
        token = auth_response_as_json.get('token').encode('ascii')
        ad_user['_id'] = auth_response_as_json['user']

        add_to_context(context, token, ad_user)


class NotificationMock:
    def __init__(self):
        self.messages = []
        self.client = None
        self.open = True

    def send(self, message):
        self.messages.append(message)

    def reset(self):
        self.messages = []


def setup_notification(context):
    mock = NotificationMock()
    if context.app.notification_client:
        mock.client = context.app.notification_client
    context.app.notification_client = mock


def teardown_notification(context):
    context.app.notification_client = context.app.notification_client.client


class TestCase(unittest.TestCase):
    def __init__(self, *a, **kw):
        super().__init__(*a, **kw)

        self.app = None
        self.client = None
        self.ctx = None

    @classmethod
    def setUpClass(cls):
        """Wrap `setUp` and `tearDown` methods to run `setUpForChildren` and `tearDownForChildren`."""
        # setUp
        def wrapper(self, *args, **kwargs):
            """Combine `setUp` with `setUpForChildren`."""
            self.setUpForChildren()
            return orig_setup(self, *args, **kwargs)
        orig_setup = cls.setUp
        cls.setUp = wrapper

        # tearDown
        def wrapper(self, *args, **kwargs):
            """Combine `tearDown` with `tearDownForChildren`."""
            self.tearDownForChildren()
            return orig_teardown(self, *args, **kwargs)
        orig_teardown = cls.tearDown
        cls.tearDown = wrapper

    def setUpForChildren(self):
        """Run this `setUp` stuff for each children."""
        setup(self)

        self.ctx = self.app.app_context()
        self.ctx.push()

        def clean_ctx():
            if self.ctx:
                self.ctx.pop()
        self.addCleanup(clean_ctx)

    def tearDownForChildren(self):
        """Run this `tearDown` stuff for each children."""

    def get_fixture_path(self, filename):
        rootpath = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
        return os.path.join(rootpath, 'features', 'steps', 'fixtures', filename)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from eve.versioning import resolve_document_version
from flask import current_app as app
import superdesk
from collections import Counter
from eve.utils import config, ParsedRequest
from eve.validation import ValidationError
from superdesk.errors import SuperdeskApiError
from superdesk import get_resource_service
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, EMBARGO
from superdesk.metadata.packages import LINKED_IN_PACKAGES, PACKAGE, \
    REFS, RESIDREF, GROUPS, ID_REF, MAIN_GROUP, ROOT_GROUP, ROLE, ROOT_ROLE, MAIN_ROLE, GROUP_ID
from apps.archive.common import insert_into_versions, ITEM_UNLINK
from apps.archive.archive import SOURCE as ARCHIVE
from superdesk.utc import utcnow
from superdesk.default_settings import VERSION
from apps.templates.content_templates import render_content_template_by_id

logger = logging.getLogger(__name__)
package_create_signal = superdesk.signals.signal('package.create')  # @UndefinedVariable


def create_root_group(docs):
    """Define groups in given docs if not present or empty.

    :param docs: list of docs
    """
    for doc in docs:
        if len(doc.get(GROUPS, [])):
            continue
        doc[GROUPS] = [
            {GROUP_ID: ROOT_GROUP, ROLE: ROOT_ROLE, REFS: [{ID_REF: MAIN_GROUP}]},
            {GROUP_ID: MAIN_GROUP, ROLE: MAIN_ROLE, REFS: []}
        ]


def get_item_ref(item):
    """Get reference for given item which can be used in group.refs.

    :param item: item dict
    """
    return {
        RESIDREF: item.get('_id'),
        'headline': item.get('headline'),
        'slugline': item.get('slugline'),
        'location': 'archive',
        'itemClass': 'icls:' + item.get('type', 'text'),
        'renditions': item.get('renditions', {})
    }


def copy_metadata_from_highlight_template(doc):
    """
    Copy the values set on highlight template

    :param doc
    """
    highlight_id = doc.get('highlight', None)
    if highlight_id:
        highlight = superdesk.get_resource_service('highlights').find_one(req=None, _id=highlight_id)
        if highlight and 'template' in highlight:
            updates = render_content_template_by_id(doc, highlight.get('template', None))
            doc.update(updates)


class PackageService():
    def on_create(self, docs):
        create_root_group(docs)
        self.check_root_group(docs)
        self.check_package_associations(docs)
        self.check_not_in_personal_space(docs)

        for doc in docs:
            if not doc.get('ingest_provider'):
                doc['source'] = app.config.get('DEFAULT_SOURCE_VALUE_FOR_MANUAL_ARTICLES')

            if 'highlight' in doc:
                copy_metadata_from_highlight_template(doc)

        package_create_signal.send(self, docs=docs)

    def on_created(self, docs):
        for (doc, assoc) in [(doc, assoc) for doc in docs
                             for assoc in self._get_associations(doc)]:
            self.update_link(doc, assoc)

    def on_update(self, updates, original):
        self.check_root_group([updates])
        associations = self._get_associations(updates)
        self.check_for_duplicates(original, associations)
        for assoc in associations:
            self.extract_default_association_data(original, assoc)

    def on_updated(self, updates, original):
        if updates.get(GROUPS):
            self.update_groups(updates, original)

    def update_groups(self, updates, original):
        to_add = {assoc.get(RESIDREF): assoc for assoc in self._get_associations(updates)
                  if assoc.get(RESIDREF) and updates.get(GROUPS)}
        to_remove = (assoc for assoc in self._get_associations(original)
                     if assoc.get(RESIDREF) not in to_add)
        for assoc in to_remove:
            self.update_link(original, assoc, delete=True)
        for assoc in to_add.keys():
            self.update_link(original, to_add[assoc])

    def on_deleted(self, doc):
        for assoc in self._get_associations(doc):
            self.update_link(doc, assoc, delete=True)

    def check_root_group(self, docs):
        for groups in [doc.get(GROUPS) for doc in docs if doc.get(GROUPS)]:
            self.check_all_groups_have_id_set(groups)
            root_groups = [group for group in groups if group.get(GROUP_ID) == ROOT_GROUP]

            if len(root_groups) == 0:
                message = 'Root group is missing.'
                logger.error(message)
                raise SuperdeskApiError.forbiddenError(message=message)

            if len(root_groups) > 1:
                message = 'Only one root group is allowed.'
                logger.error(message)
                raise SuperdeskApiError.forbiddenError(message=message)

            self.check_that_all_groups_are_referenced_in_root(root_groups[0], groups)

    def check_all_groups_have_id_set(self, groups):
        if any(group for group in groups if not group.get(GROUP_ID)):
            message = 'Group is missing id.'
            logger.error(message)
            raise SuperdeskApiError.forbiddenError(message=message)

    def check_that_all_groups_are_referenced_in_root(self, root, groups):
        rest = [group.get(GROUP_ID) for group in groups if group.get(GROUP_ID) != ROOT_GROUP]
        refs = [ref.get(ID_REF) for group in groups for ref in group.get(REFS, [])
                if ref.get(ID_REF)]

        rest_counter = Counter(rest)
        if any(id for id, value in rest_counter.items() if value > 1):
            message = '{id} group is added multiple times.'.format(id=id)
            logger.error(message)
            raise SuperdeskApiError.forbiddenError(message=message)

        if len(rest) != len(refs):
            message = 'The number of groups and of referenced groups in the root group do not match.'
            logger.error(message)
            raise SuperdeskApiError.forbiddenError(message=message)

        if len(set(rest).intersection(refs)) != len(refs):
            message = 'Not all groups are referenced in the root group.'
            logger.error(message)
            raise SuperdeskApiError.forbiddenError(message=message)

    def check_package_associations(self, docs):
        for (doc, group) in ((doc, group) for doc in docs for group in doc.get(GROUPS, [])):
            associations = group.get(REFS, [])
            self.check_for_duplicates(doc, associations)
            for assoc in associations:
                self.extract_default_association_data(group, assoc)

    def check_not_in_personal_space(self, docs):
        """Verify that the package is not in the user personal space.

        Retrieving details for the list of packages an item was linked in does not
        contain packages that were in other users' personal space. One can't spike
        items if it can't remove it from the packages it belongs to.
        """
        for doc in docs:
            # the next 'particular_type' containing line is liveblog related and it is usefull since
            # liveblog users don't have a personal space. Therefore, if the the task/desk condition
            # is in place, it prevents the liveblog posts from beeing published.
            if not doc.get('particular_type'):
                if not doc.get('task') or not doc['task'].get('desk'):
                    message = 'Packages can not be created in the personal space.'
                    logger.error(message)
                    raise SuperdeskApiError.forbiddenError(message=message)
                if not doc['task'].get('stage'):
                    desk = get_resource_service('desks').find_one(req=None, _id=doc['task']['desk'])
                    doc['task']['stage'] = desk['working_stage']

    def extract_default_association_data(self, package, assoc):
        if assoc.get(ID_REF):
            return

        item, item_id, endpoint = self.get_associated_item(assoc)
        self.check_for_circular_reference(package, item_id)
        assoc['guid'] = item.get('guid', item_id)
        assoc['type'] = item.get('type')

    def get_associated_item(self, assoc, throw_if_not_found=True):
        endpoint = assoc.get('location', 'archive')
        item_id = assoc[RESIDREF]

        if not item_id:
            raise SuperdeskApiError.badRequestError("Package contains empty ResidRef!")

        item = get_resource_service(endpoint).find_one(req=None, _id=item_id)

        if not item and throw_if_not_found:
            message = 'Invalid item reference: ' + assoc[RESIDREF]
            logger.error(message)
            raise SuperdeskApiError.notFoundError(message=message)
        return item, item_id, endpoint

    def update_link(self, package, assoc, delete=False):
        # skip root node
        if assoc.get(ID_REF):
            return
        package_id = package[config.ID_FIELD]

        item, item_id, endpoint = self.get_associated_item(assoc, not delete)
        if not item and delete:
            # just exit, no point on complaining
            return

        two_way_links = [d for d in item.get(LINKED_IN_PACKAGES, []) if not d['package'] == package_id]

        if not delete:
            data = {PACKAGE: package_id}
            two_way_links.append(data)

        updates = {LINKED_IN_PACKAGES: two_way_links}
        get_resource_service(endpoint).system_update(item_id, updates, item)

    def check_for_duplicates(self, package, associations):
        counter = Counter()
        package_id = package[config.ID_FIELD]
        for itemRef in [assoc[RESIDREF] for assoc in associations if assoc.get(RESIDREF)]:
            if itemRef == package_id:
                message = 'Trying to self reference as an association.'
                logger.error(message)
                raise SuperdeskApiError.forbiddenError(message=message)
            counter[itemRef] += 1

        if any(itemRef for itemRef, value in counter.items() if value > 1):
            message = 'Content associated multiple times'
            logger.error(message)
            raise SuperdeskApiError.forbiddenError(message=message)

    def check_for_circular_reference(self, package, item_id):
        if any(d for d in package.get(LINKED_IN_PACKAGES, []) if d['package'] == item_id):
            message = 'Trying to create a circular reference to: ' + item_id
            logger.error(message)
            raise ValidationError(message)
        else:
            # keep checking in the hierarchy
            for d in (d for d in package.get(LINKED_IN_PACKAGES, []) if 'package' in d):
                linked_package = get_resource_service(ARCHIVE).find_one(req=None, _id=d['package'])
                if linked_package:
                    self.check_for_circular_reference(linked_package, item_id)

    def get_packages(self, doc_id, not_package_id=None):
        """
        Retrieves package(s) if an article identified by doc_id is referenced in a package.

        :param str doc_id: identifier of the item in the package
        :param str not_package_id: not package id
        :return: articles of type composite
        """

        query = {'$and': [{ITEM_TYPE: CONTENT_TYPE.COMPOSITE}, {'groups.refs.residRef': doc_id}]}

        if not_package_id:
            query['$and'].append({config.ID_FIELD: {'$ne': not_package_id}})

        request = ParsedRequest()
        request.max_results = 100

        return get_resource_service(ARCHIVE).get_from_mongo(req=request, lookup=query)

    def remove_ref_from_inmem_package(self, package, ref_id):
        """Removes the reference with ref_id from non-root groups.

        If there is nothing left
        in that group then the group and its reference in root group is also removed.
        If the removed item was the last item then returns

        :param package: Package
        :param ref_id: Id of the reference to be removed
        :return: True if there are still references in the package, False otherwise
        """
        groups_to_be_removed = set()
        non_root_groups = [group for group in package.get(GROUPS, []) if group.get(GROUP_ID) != ROOT_GROUP]
        for non_rg in non_root_groups:
            refs = [r for r in non_rg.get(REFS, []) if r.get(RESIDREF, '') != ref_id]
            if len(refs) == 0:
                groups_to_be_removed.add(non_rg.get(GROUP_ID))
            non_rg[REFS] = refs

        if len(groups_to_be_removed) > 0:
            root_group = [group for group in package.get(GROUPS, []) if group.get(GROUP_ID) == ROOT_GROUP][0]
            refs = [r for r in root_group.get(REFS, []) if r.get(ID_REF) not in groups_to_be_removed]
            root_group[REFS] = refs
            removed_groups = [group for group in package.get(GROUPS, [])
                              if group.get(GROUP_ID) not in groups_to_be_removed]
            package[GROUPS] = removed_groups

            # return if the package has any items left in it
            return len(refs) > 0

        # still has items in the package
        return True

    def replace_ref_in_package(self, package, old_ref_id, new_ref_id):
        """Locates the reference with the old_ref_id and replaces with the new_ref_id

        :param package: Package
        :param old_ref_id: Old reference id
        :param new_ref_id: New reference id
        """
        non_root_groups = (group for group in package.get(GROUPS, []) if group.get(GROUP_ID) != ROOT_GROUP)
        for g in (ref for group in non_root_groups for ref in group.get(REFS, [])):
            if g.get(RESIDREF, '') == old_ref_id:
                new_item = get_resource_service('archive').find_one(req=None, _id=new_ref_id)
                g[RESIDREF] = new_ref_id
                g['guid'] = new_ref_id
                g[VERSION] = new_item[VERSION]

    def update_field_in_package(self, package, ref_id, field, field_value):
        """Locates the reference with the ref_id and replaces field value

        :param package: Package
        :param ref_id: reference id
        :param field: field to be replaced
        :param field_value: value to be used
        """
        non_root_groups = (group for group in package.get(GROUPS, []) if group.get(GROUP_ID) != ROOT_GROUP)
        for g in (ref for group in non_root_groups for ref in group.get(REFS, [])):
            if g.get(RESIDREF, '') == ref_id:
                g[field] = field_value

    def remove_group_ref(self, package, ref_id):
        groups = package[GROUPS]
        new_groups = [{GROUP_ID: group[GROUP_ID], ROLE: group.get(ROLE),
                       REFS: [ref for ref in group[REFS] if ref.get('guid') != ref_id]}
                      for group in groups]
        new_root_refs = [{ID_REF: group[GROUP_ID]} for group in new_groups if group[GROUP_ID] != ROOT_GROUP]

        for group in new_groups:
            if group[GROUP_ID] == ROOT_GROUP:
                group[REFS] = new_root_refs
                break

        return new_groups

    def remove_refs_in_package(self, package, ref_id_to_remove, processed_packages=None):
        """Removes residRef referenced by ref_id_to_remove from the package associations and returns the package id.

        Before removing checks if the package has been processed. If processed the package is skipped.

        :return: package[config.ID_FIELD]
        """
        groups = package[GROUPS]

        if processed_packages is None:
            processed_packages = []

        sub_package_ids = [ref['guid'] for group in groups
                           for ref in group[REFS] if ref.get('type') == CONTENT_TYPE.COMPOSITE]
        for sub_package_id in sub_package_ids:
            if sub_package_id not in processed_packages:
                sub_package = self.find_one(req=None, _id=sub_package_id)
                return self.remove_refs_in_package(sub_package, ref_id_to_remove)

        new_groups = self.remove_group_ref(package, ref_id_to_remove)
        updates = {config.LAST_UPDATED: utcnow(), GROUPS: new_groups}

        resolve_document_version(updates, ARCHIVE, 'PATCH', package)
        get_resource_service(ARCHIVE).patch(package[config.ID_FIELD], updates)
        app.on_archive_item_updated(updates, package, ITEM_UNLINK)
        insert_into_versions(id_=package[config.ID_FIELD])

        sub_package_ids.append(package[config.ID_FIELD])
        return sub_package_ids

    def _get_associations(self, doc):
        return [assoc for group in doc.get(GROUPS, [])
                for assoc in group.get(REFS, [])]

    def remove_spiked_refs_from_package(self, doc_id, not_package_id=None):
        packages = self.get_packages(doc_id, not_package_id)
        if packages.count() == 0:
            return

        processed_packages = []
        for package in packages:
            if str(package[config.ID_FIELD]) in processed_packages:
                continue

            processed_packages.extend(self.remove_refs_in_package(package, doc_id, processed_packages))

    def get_residrefs(self, package):
        """
        Returns all residref in the package.

        :param package:
        :return: list of residref
        """
        return [ref.get(RESIDREF) for group in package.get(GROUPS, [])
                for ref in group.get(REFS, []) if RESIDREF in ref]

    def check_if_any_item_in_package_has_embargo(self, package):
        """Recursively checks if any item in the package has embargo.

        :raises: SuperdeskApiError.badRequestError() if any item in the package has embargo.
        """

        item_refs_in_package = self.get_residrefs(package)

        for item_ref in item_refs_in_package:
            doc = get_resource_service(ARCHIVE).find_one(req=None, _id=item_ref)

            if doc.get(EMBARGO):
                raise SuperdeskApiError.badRequestError("Package can't have item which has embargo. "
                                                        "Slugline/Unique Name of the item having embargo: %s/%s" %
                                                        (doc.get('slugline'), doc.get('unique_name')))

            if doc[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
                self.check_if_any_item_in_package_has_embargo(doc)

    def get_item_refs(self, package):
        """Returns all item references in the package.

        :param package:
        :return: list of item references
        """

        return [ref for group in package.get(GROUPS, []) for ref in group.get(REFS, []) if RESIDREF in ref]

    def get_linked_in_package_ids(self, item):
        """Returns all linked in package ids for an item

        :param dict item:
        :return list: list of package ids
        """
        return [package_link.get(PACKAGE) for package_link in item.get(LINKED_IN_PACKAGES, []) or []]

    def get_linked_in_packages(self, item):
        """Returns all linked in packages for an item

        :param dict item:
        :return list: list of package ids
        """

        return [package for package in item.get(LINKED_IN_PACKAGES, [])]

    def is_package(self, item):
        """Test if given item is a package

        :param dict item: item to check
        """
        return item[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE



"""
Superdesk Google Authentication

.. versionadded:: 1.8

You can use Google for authentication, first you have to create credentials
in `Google API console <https://console.developers.google.com/apis/credentials>`_:

- set your client URL as *Authorized JavaScript origins*::

    https://example.com

- set server URL + ``/api/login/google_authorized`` as *Authorized redirect URIs*::

    https://example.com/api/login/google_authorized

Once configured you will find there *Client ID* and *Client secret*, use both to populate :ref:`settings.google_oauth`.

.. versionchanged:: 1.9
    There is no need to configure client, it reads config from server now.

.. versionchanged:: 1.9
    Login url is ``/api/login/google_authorized`` instead of ``/login/google_authorized``.

"""

import superdesk

from flask import url_for, session
from flask_oauthlib.client import OAuth

from superdesk.auth import auth_user


bp = superdesk.Blueprint('oauth', __name__)


def init_app(app):
    oauth = OAuth(app)
    app.client_config['google_auth'] = False
    if app.config.get('GOOGLE_CLIENT_ID') and app.config.get('GOOGLE_CLIENT_SECRET'):
        app.client_config['google_auth'] = True
        configure_google(app, oauth)
        superdesk.blueprint(bp, app)


def configure_google(app, oauth):
    google = oauth.remote_app(
        'google',
        consumer_key=app.config.get('GOOGLE_CLIENT_ID'),
        consumer_secret=app.config.get('GOOGLE_CLIENT_SECRET'),
        request_token_params={'scope': 'email'},

        base_url='https://www.googleapis.com/oauth2/v1/',
        request_token_url=None,
        access_token_method='POST',
        access_token_url='https://accounts.google.com/o/oauth2/token',
        authorize_url='https://accounts.google.com/o/oauth2/auth',
    )

    @bp.route('/login/google')
    def google_login():
        return google.authorize(callback=url_for('oauth.google_authorized', _external=True))

    @bp.route('/login/google_authorized')
    def google_authorized():
        resp = google.authorized_response()
        if resp is None:
            return auth_user()
        session['google_token'] = (resp['access_token'], '')
        me = google.get('userinfo')
        return auth_user(me.data['email'])

    @google.tokengetter
    def get_google_oauth_token():
        return session.get('google_token')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import get_backend
from apps.publish.formatters.service import FormattersService
from apps.publish.formatters.resource import FormattersResource


def init_app(app):
    endpoint_name = 'formatters'
    service = FormattersService(endpoint_name, backend=get_backend())
    FormattersResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk IO Registry"""

from superdesk import Resource, Service
from superdesk.utils import ListCursor
from superdesk.errors import SuperdeskIngestError, AlreadyExistsError

registered_feed_parsers = {}
allowed_feed_parsers = []

registered_feeding_services = {}
allowed_feeding_services = []
feeding_service_errors = {}
publish_errors = []


def register_feeding_service(service_name, service_class, errors):
    """
    Registers the Feeding Service with the application.

    :class: `superdesk.io.feeding_services.RegisterFeedingService` uses this function to register the feeding service.

    :param service_name: unique name to identify the Feeding Service class
    :param service_class: Feeding Service class
    :param errors: list of tuples, where each tuple represents an error that can be raised by a Feeding Service class.
                   Tuple syntax: (error_code, error_message)
    :raises: AlreadyExistsError if a feeding service with same name already been registered
    """

    if service_name in registered_feeding_services:
        raise AlreadyExistsError('Feeding Service: {} already registered by {}'
                                 .format(service_name, type(registered_feeding_services[service_name])))

    registered_feeding_services[service_name] = service_class
    allowed_feeding_services.append(service_name)

    errors.append(SuperdeskIngestError.parserNotFoundError().get_error_description())
    feeding_service_errors[service_name] = dict(errors)


def register_feeding_service_error(service_name, error):
    """
    Registers an error with the service named

    :param service_name: unique name to identify the Feeding Service class
    :param error: tuple representing the error, the tuple contains the error_code and the error message
    :return:
    """
    feeding_service_errors.get(service_name, {}).update(dict([error]))


def register_feed_parser(parser_name, parser_class):
    """
    Registers the Feed Parser with the application.

    :class: `superdesk.io.feed_parsers.RegisterFeedParser` uses this function to register the feed parser.

    :param parser_name: unique name to identify the Feed Parser class
    :param parser_class: Feed Parser class
    :raises: AlreadyExistsError if a feed parser with same name already been registered
    """

    if parser_name in registered_feed_parsers:
        raise AlreadyExistsError('Feed Parser: {} already registered by {}'
                                 .format(parser_name, type(registered_feed_parsers[parser_name])))

    registered_feed_parsers[parser_name] = parser_class
    allowed_feed_parsers.append(parser_name)


def get_feed_parser(parser_name):
    """
    Retrieve registered feed parser class from its name

    :param parser_name: name of the parser, as in register_feed_parser
    :type parser_name: str
    :return str: feed parser class
    :raise KeyError: there is no parser registered with this name
    """
    return registered_feed_parsers[parser_name]


class FeedParserAllowedResource(Resource):
    resource_methods = ['GET']
    item_methods = []


class FeedParserAllowedService(Service):

    def get(self, req, lookup):
        def parser(parser_id):
            registered = registered_feed_parsers[parser_id]
            return {
                'feed_parser': parser_id,
                'label': getattr(registered, 'label', parser_id)
            }

        return ListCursor(
            [parser(_id) for _id in registered_feed_parsers]
        )


class FeedingServiceAllowedResource(Resource):
    resource_methods = ['GET']
    item_methods = []


class FeedingServiceAllowedService(Service):

    def get(self, req, lookup):
        def service(service_id):
            registered = registered_feeding_services[service_id]
            return {
                'feeding_service': service_id,
                'label': getattr(registered, 'label', service_id)
            }

        return ListCursor(
            [service(_id) for _id in registered_feeding_services]
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from superdesk.services import BaseService
import superdesk.metadata.utils
import datetime
import superdesk


class DataUpdatesResource(Resource):
    schema = {
        'name': {
            'type': 'string',
            'required': True
        },
        'applied': {
            'type': 'datetime',
            'required': True
        }
    }
    internal_resource = True
    item_url = superdesk.metadata.utils.item_url


class DataUpdatesService(BaseService):
    def on_create(self, docs):
        for doc in docs:
            doc['applied'] = datetime.datetime.now()


def init_app(app):
    endpoint_name = 'data_updates'
    service = DataUpdatesService(endpoint_name, backend=superdesk.get_backend())
    DataUpdatesResource(endpoint_name, app=app, service=service)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from superdesk.logging import logger
from superdesk.utc import utcnow
from superdesk.notification import push_notification
from apps.content import push_expired_notification
from superdesk.errors import ProviderError
from superdesk.lock import lock, unlock
from superdesk.io.registry import registered_feeding_services


class RemoveExpiredContent(superdesk.Command):
    """Remove stale data from ingest based on the provider settings."""

    option_list = (
        superdesk.Option('--provider', '-p', dest='provider_name'),
    )

    def run(self, provider_name=None):
        providers = list(superdesk.get_resource_service('ingest_providers').get(req=None, lookup={}))
        self.remove_expired({'exclude': [str(p.get('_id')) for p in providers]})
        for provider in providers:
            if not provider_name or provider_name == provider.get('name'):
                self.remove_expired(provider)

    def remove_expired(self, provider):
        lock_name = 'ingest:gc'

        if not lock(lock_name, expire=300):
            return

        try:

            remove_expired_data(provider)
            push_notification('ingest:cleaned')
        except Exception as err:
            logger.exception(err)
            raise ProviderError.expiredContentError(err, provider)
        finally:
            unlock(lock_name)


superdesk.command('ingest:clean_expired', RemoveExpiredContent())


def remove_expired_data(provider):
    """Remove expired data for provider"""
    logger.info('Removing expired content for provider: %s' % provider.get('_id', 'Detached items'))

    try:
        feeding_service = registered_feeding_services[provider['feeding_service']]
        feeding_service = feeding_service.__class__()
        ingest_collection = feeding_service.service if hasattr(feeding_service, 'service') else 'ingest'
    except KeyError:
        ingest_collection = 'ingest'

    ingest_service = superdesk.get_resource_service(ingest_collection)

    items = get_expired_items(provider, ingest_collection)

    ids = [item['_id'] for item in items]
    items.rewind()
    file_ids = [rend.get('media')
                for item in items
                for rend in item.get('renditions', {}).values()
                if not item.get('archived') and rend.get('media')]

    if ids:
        logger.info('Removing items %s' % ids)
        ingest_service.delete({'_id': {'$in': ids}})
        push_expired_notification(ids)

    for file_id in file_ids:
        logger.info('Deleting file: %s' % file_id)
        superdesk.app.media.delete(file_id)

    logger.info('Removed expired content for provider: {0} count: {1}'
                .format(provider.get('_id', 'Detached items'), len(ids)))

    remove_expired_from_elastic(ingest_collection)


def remove_expired_from_elastic(ingest_collection):
    """Remove expired items from elastic which shouldn't be there anymore - expired before previous run."""
    ingest = superdesk.get_resource_service(ingest_collection)
    items = ingest.search({'filter': {'range': {'expiry': {'lt': 'now-5m/m'}}}})
    if items.count():
        logger.warning('there are expired items in elastic (%d)' % (items.count(), ))
        for item in items:
            logger.debug('doc only in elastic item=%s' % (item, ))
            ingest.remove_from_search(item)


def get_expired_items(provider_id, ingest_collection):
    query_filter = get_query_for_expired_items(provider_id)
    return superdesk.get_resource_service(ingest_collection).get_from_mongo(lookup=query_filter, req=None)


def get_query_for_expired_items(provider):
    """Find all ingest items with given provider id and expiry is past

    :param dict provider: ingest provider
    :return str: mongo query
    """
    query = {'expiry': {'$lte': utcnow()}}

    if provider.get('_id'):
        query['ingest_provider'] = str(provider.get('_id'))

    if provider.get('exclude'):
        excluded = provider.get('exclude')
        query['ingest_provider'] = {'$nin': excluded}

    return query



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from abc import ABCMeta, abstractmethod
from datetime import datetime

from pytz import utc

import superdesk
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, SuperdeskIngestError
from superdesk.io.registry import registered_feed_parsers
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)


class FeedingService(metaclass=ABCMeta):
    """
    Base Class for all Feeding Service classes.

    A Feeding Service class must have the following attributes:
        1. `NAME` - unique name under which to register the class.
        2. `ERRORS` - representing a list of <error_number, error_message> pairs that might be raised by the class
                      instances' methods.
    """

    @abstractmethod
    def _update(self, provider, update):
        """
        Subclasses must override this method and get items from the provider as per the configuration.

        :param provider: Ingest Provider Details.
                .. seealso:: :class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :type provider: dict
        :param update: Any update that is required on provider.
        :type update: dict
        :return: a list of articles which can be saved in Ingest Collection.
        """
        raise NotImplementedError()

    def _test(self, provider):
        """
        Subclasses should override this method and do specific config test.

        :param provider: Ingest Provider Details.
        """
        return

    def config_test(self, provider=None):
        """Test provider configuration.

        :param provider: provider data
        """
        if not provider:  # nosetests run this for some reason
            return
        if self._is_closed(provider):
            return
        return self._test(provider)

    def _is_closed(self, provider):
        """Test if provider is closed.

        :param provider: provider data
        :return bool: True if is closed
        """
        is_closed = provider.get('is_closed', False)

        if isinstance(is_closed, datetime):
            is_closed = False

        return is_closed

    def update(self, provider, update):
        """
        Clients consuming Ingest Services should invoke this to get items from the provider.

        :param provider: Ingest Provider Details.
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :param update: Any update that is required on provider.
        :type update: dict
        :return: a list of articles which can be saved in Ingest Collection.
        :raises SuperdeskApiError.internalError if Provider is closed
        :raises SuperdeskIngestError if failed to get items from provider
        """
        if self._is_closed(provider):
            raise SuperdeskApiError.internalError('Ingest Provider is closed')
        else:
            try:
                return self._update(provider, update) or []
            except SuperdeskIngestError as error:
                self.close_provider(provider, error)
                raise error

    def close_provider(self, provider, error, force=False):
        """Closes the provider and uses error as reason for closing.

        :param provider: Ingest Provider Details.
                .. seealso:: :class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :param error: ingest error
        :param force: force closing of provider, no matter how it's configured
        """
        if provider.get('critical_errors', {}).get(str(error.code)) or force:
            updates = {
                'is_closed': True,
                'last_closed': {
                    'closed_at': utcnow(),
                    'message': 'Channel closed due to critical error: {}'.format(error)
                }
            }

            get_resource_service('ingest_providers').system_update(provider[superdesk.config.ID_FIELD],
                                                                   updates, provider)

    def add_timestamps(self, item):
        """
        Adds firstcreated and versioncreated timestamps to item

        :param item: object which can be saved to ingest collection
        :type item: dict
        """

        item['firstcreated'] = utc.localize(item['firstcreated']) if item.get('firstcreated') else utcnow()
        item['versioncreated'] = utc.localize(item['versioncreated']) if item.get('versioncreated') else utcnow()

    def log_item_error(self, err, item, provider):
        """TODO: put item into provider error basket."""
        logger.warning('ingest error msg={} item={} provider={}'.format(
            str(err),
            item.get('guid'),
            provider.get('name')
        ))

    def prepare_href(self, href, mimetype=None):
        """Prepare a link to an external resource (e.g. an image file).

        It can be directly used by the ingest machinery for fetching it.

        Sub-classes can override this method if properties like HTTP Authentication need to be added to the href.

        :param href: the original URL as extracted from an RSS entry
        :type href: str
        :return: prepared URL
        :rtype: str
        """
        return href

    def get_feed_parser(self, provider, article=None):
        """
        Returns instance of configured feed parser for the given provider.

        :param provider: Ingest Provider Details.
        :type provider: dict :py:class: `superdesk.io.ingest_provider_model.IngestProviderResource`
        :param article: article which needs to parsed by the parser, defaults to None. When None, the registered parser
                        will be returned without checking if the parser can parse the article.
        :return: Feed Parser instance.
        :rtype: :py:class:`superdesk.io.feed_parsers.FeedParser`
        :raises: SuperdeskIngestError.parserNotFoundError()
                    if either feed_parser value is empty or Feed Parser not found.
        """

        parser = registered_feed_parsers.get(provider.get('feed_parser', ''))
        if not parser:
            raise SuperdeskIngestError.parserNotFoundError(provider=provider)

        if article is not None and not parser.can_parse(article):
            raise SuperdeskIngestError.parserNotFoundError(provider=provider)

        if article is not None:
            parser = parser.__class__()

        return parser


# must be imported for registration
from superdesk.io.feeding_services.email import EmailFeedingService  # NOQA
from superdesk.io.feeding_services.file_service import FileFeedingService  # NOQA
from superdesk.io.feeding_services.ftp import FTPFeedingService  # NOQA
from superdesk.io.feeding_services.ritzau import RitzauFeedingService  # NOQA
from superdesk.io.feeding_services.http_service import HTTPFeedingService  # NOQA
from superdesk.io.feeding_services.rss import RSSFeedingService  # NOQA
from superdesk.io.feeding_services.twitter import TwitterFeedingService  # NOQA
from superdesk.io.feeding_services.ap import APFeedingService  # NOQA



import json
import string
import random

from eve.utils import ParsedRequest
from eve.io.mongo import MongoJSONEncoder

from apps.dictionaries.service import words, DictionaryService, store_dict, fetch_dict, FILE_ID
from superdesk import get_resource_service
from superdesk.tests import TestCase


class WordsTestCase(TestCase):

    def setUp(self):
        self.req = ParsedRequest()
        with self.app.test_request_context(self.app.config.get('URL_PREFIX')):
            self.dictionaries = [{'_id': '1', 'name': 'Eng', 'language_id': 'en'},
                                 {'_id': '2', 'name': 'Eng AUs', 'language_id': 'en-AU', 'is_active': 'true'},
                                 {'_id': '3', 'name': 'French', 'language_id': 'fr'}]
            self.app.data.insert('dictionaries', self.dictionaries)

    def test_words_parsing(self):
        self.assertEquals(['abc'], words('abc'))
        self.assertEqual(['abc bce'], words('abc bce'))
        self.assertEqual(['abc bce', 'wer tyu'], words('abc bce\nwer tyu'))
        self.assertEqual(['you\'d'], words('you\'d'))
        self.assertEqual(['you\'d', 'will'], words('you\'d\nwill'))

    def test_base_language(self):
        self.assertEqual(DictionaryService().get_base_language('en-AU'), 'en')
        self.assertIsNone(DictionaryService().get_base_language('en'))

    def test_get_dictionary(self):
        with self.app.app_context():
            dicts = get_resource_service('dictionaries').get_dictionaries('en')
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]['language_id'], 'en')
            dicts = get_resource_service('dictionaries').get_dictionaries('en-AU')
            self.assertEqual(len(dicts), 1)
            self.assertEqual(dicts[0]['language_id'], 'en-AU')

    def test_store_dict_small(self):
        content = {'foo': 1}
        dictionary = {'content': content}
        store_dict(dictionary, {})
        self.assertEqual(json.dumps(content), dictionary.get('content'))
        self.assertEqual(content, fetch_dict(dictionary))

    def test_store_dict_big(self):
        content = self._get_big_dict()
        dictionary = {'content': content}
        self.assertGreater(len(json.dumps(dictionary, cls=MongoJSONEncoder)), 1000000)
        store_dict(dictionary, {})
        self.assertIsNone(dictionary['content'])
        self.assertEqual(content, fetch_dict(dictionary))
        self.assertLess(len(json.dumps(dictionary, cls=MongoJSONEncoder)), 100)

    def test_store_big_after_small(self):
        original = {'content': {'foo': 1}}
        updates = {'content': self._get_big_dict()}
        store_dict(updates, original)
        self.assertIsNone(updates['content'])
        self.assertIsNotNone(updates[FILE_ID])

    def test_small_after_big(self):
        updates = {'content': self._get_big_dict()}
        store_dict(updates, {})
        file_id = updates[FILE_ID]
        self.assertIsNotNone(self.app.storage.get(file_id))

        original = updates
        updates = {'content': {'foo': 1}}
        store_dict(updates, original)

        self.assertIsNone(updates[FILE_ID])
        self.assertEqual(json.dumps({'foo': 1}), updates['content'])
        self.assertIsNone(self.app.storage.get(file_id))

    def _get_big_dict(self):
        word = ''.join([random.choice(string.ascii_letters) for i in range(1000000)])
        return {word: 1}



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.io.registry import register_feed_parser
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE
from superdesk.errors import ParserError
from superdesk import etree as sd_etree
import superdesk
import dateutil.parser
import logging
from pytz import timezone

logger = logging.getLogger(__name__)
NS = {'r': 'http://tempuri.org/',
      'a': 'http://schemas.microsoft.com/2003/10/Serialization/Arrays'}


class RitzauFeedParser(XMLFeedParser):
    """
    Feed Parser which can parse Ritzau XML feed
    """

    _subjects_map = None

    NAME = 'ritzau'
    label = "Ritzau feed"

    def __init__(self):
        super().__init__()

        self.default_mapping = {
            'guid': 'NewsID/text()',
            'body_html': {'xpath': 'content/text()',
                          'filter': sd_etree.clean_html_str},
            'firstcreated': {'xpath': 'PublishDate/text()',
                             'filter': self._publish_date_filter,
                             },
            'versioncreated': {'xpath': 'PublishDate/text()',
                               'filter': self._publish_date_filter,
                               },
            'headline': 'headline/text()',
            'priority': 'Priority/text()',
            'keywords': {'xpath': 'strapline/text()',
                         'filter': lambda v: list(filter(None, v.split('/')))},
            'abstract': 'subtitle',
            'byline': 'origin',
            'version': {'xpath': 'version/text()',
                        'filter': int},
            'ednote': 'Tilredaktionen/text()',
            'subject': {'xpath': 'IPTCList/a:int/text()',
                        'list': True,
                        'filter': self._subject_filter}}

    @property
    def subjects_map(self):
        if self._subjects_map is None:
            voc_subjects = superdesk.get_resource_service('vocabularies').find_one(req=None, _id='subject_custom')
            if voc_subjects is not None:
                self._subjects_map = {i['qcode']: i for i in voc_subjects['items']}
            else:
                self._subjects_map = {}
        return self._subjects_map

    def can_parse(self, xml):
        return xml.tag.endswith('RBNews')

    def parse(self, xml, provider=None):
        item = {ITEM_TYPE: CONTENT_TYPE.TEXT,  # set the default type.
                }
        try:
            self.do_mapping(item, xml, namespaces=NS)
        except Exception as ex:
            raise ParserError.parseMessageError(ex, provider)
        return item

    def get_datetime(self, value):
        return dateutil.parser.parse(value)

    def _subject_filter(self, qcode):
        try:
            subject = self.subjects_map[qcode]
        except KeyError:
            return None
        else:
            if not subject.get('is_active', False):
                return None
            name = subject.get('name', '')

        return {'qcode': qcode, 'name': name, 'scheme': 'subject_custom'}

    def _publish_date_filter(self, date_string):
        dt = dateutil.parser.parse(date_string)
        return dt.replace(tzinfo=timezone('CET'))


register_feed_parser(RitzauFeedParser.NAME, RitzauFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import re

from datetime import datetime

from superdesk.errors import ParserError
from superdesk.io.registry import register_feed_parser
from superdesk.io.feed_parsers import FileFeedParser
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, Priority, GUID_FIELD, GUID_TAG, FORMAT, FORMATS
from superdesk.utc import utc
from superdesk.metadata.utils import generate_guid


class ANPAFeedParser(FileFeedParser):
    """
    Feed Parser which can parse if the feed is in ANPA 1312 format.
    """

    NAME = 'anpa1312'

    def can_parse(self, file_path):
        try:
            with open(file_path, 'rb') as f:
                lines = [line for line in f]
                return re.match(b'\x16\x16\x01([a-z])([0-9]{4})\x1f([a-z0-9-]+)', lines[0], flags=re.I)
        except Exception:
            return False

    def parse(self, file_path, provider=None):
        try:
            item = {ITEM_TYPE: CONTENT_TYPE.TEXT, GUID_FIELD: generate_guid(type=GUID_TAG), FORMAT: FORMATS.HTML}

            with open(file_path, 'rb') as f:
                lines = [line for line in f]

            # parse first header line
            m = re.match(b'\x16\x16\x01([a-z])([0-9]{4})\x1f([a-z-]+)', lines[0], flags=re.I)
            if m:
                item['provider_sequence'] = m.group(2).decode()

            # parse second header line
            m = re.match(
                b'([a-z]) ([a-z])(\x13|\x14)(\x11|\x12) (am-|pm-|bc-|ap-)([a-z-.]+)(.*) '
                b'([0-9]{1,2})-([0-9]{1,2}) ([0-9]{4})',
                lines[1], flags=re.I)
            if m:
                item['priority'] = self.map_priority(m.group(1).decode())
                item['anpa_category'] = [{'qcode': m.group(2).decode()}]
                item['slugline'] = m.group(6).decode('latin-1', 'replace')
                item['anpa_take_key'] = m.group(7).decode('latin-1', 'replace').strip()
                item['word_count'] = int(m.group(10).decode())
                if m.group(4) == b'\x12':
                    item[FORMAT] = FORMATS.PRESERVED

            # parse created date at the end of file
            m = re.search(b'\x03([a-z]+)-([a-z]+)-([0-9]+-[0-9]+-[0-9]+ [0-9]{2}[0-9]{2})GMT', lines[-4], flags=re.I)
            if m:
                item['firstcreated'] = datetime.strptime(m.group(3).decode(), '%m-%d-%y %H%M').replace(tzinfo=utc)
                item['versioncreated'] = item['firstcreated']

            # parse anpa content
            body = b''.join(lines[2:])
            m = re.match(b'\x02(.*)\x03', body, flags=re.M + re.S)
            if m:
                text = m.group(1).decode('latin-1', 'replace').split('\n')

                if item.get(FORMAT) == FORMATS.PRESERVED:
                    # ANPA defines a number of special characters e.g. TLI (Tab Line Inicator) Hex x08 and
                    # TTS Space Band Hex x10 These will be replaced, there will likely be others
                    body_lines = [l.strip('^').replace('\b', '%08').replace('\x10', '%10') for l in text if
                                  l.startswith(('\t', '^', '\b'))]
                    item['body_html'] = '<pre>' + '\n'.join(body_lines) + '</pre>'
                else:
                    body_lines = [l.strip() for l in text if l.startswith(('\t'))]
                    item['body_html'] = '<p>' + '</p><p>'.join(body_lines) + '</p>'

                # content metadata
                header_lines = [l.strip('^<= ') for l in text if l.startswith('^')]
                if len(header_lines) > 1:
                    item['headline'] = header_lines[1].rstrip('\r\n^<= ')
                if len(header_lines) > 3:
                    item['byline'] = header_lines[-2].rstrip('\r\n^<= ')

                    # if there is no body use header lines
                    if len(body_lines) == 1 and not body_lines[0]:
                        item['body_html'] = '<p>' + '</p><p>'.join(header_lines[2:]) + '</p>'

                # slugline
                if len(header_lines) > 1:
                    m = re.match('[A-Z]{2}-[A-Z]{2}--([a-z-0-9.]+)', header_lines[0], flags=re.I)
                    if m:
                        item['slugline'] = m.group(1)

                # ednote
                self._parse_ednote(header_lines, item)

            return item
        except Exception as ex:
            raise ParserError.anpaParseFileError(file_path, ex)

    def _parse_ednote(self, header_lines, item):
        for line in header_lines:
            m = re.search("EDITOR'S NOTE _(.*)", line)
            if m:
                item['ednote'] = m.group(1).strip()

    def map_priority(self, source_priority):
        mapping = {
            'f': Priority.Flash.value,
            'u': Priority.Urgent.value,
            'b': Priority.Three_Paragraph.value,
            'z': Priority.Ordinary.value
        }

        source_priority = source_priority.lower().strip() if isinstance(source_priority, str) else ''
        return mapping.get(source_priority, Priority.Ordinary.value)


register_feed_parser(ANPAFeedParser.NAME, ANPAFeedParser())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.resource import Resource
from content_api import MONGO_PREFIX


class CompanyTokenResource(Resource):
    schema = {
        '_id': {'type': 'string', 'unique': True},
        'expiry': {'type': 'datetime'},
        'company': Resource.rel('companies', required=False),
    }

    item_url = 'regex(".+")'
    resource_methods = ['GET', 'POST']
    item_methods = ['GET', 'DELETE']
    privileges = {'POST': 'subscribers', 'DELETE': 'subscribers'}

    datasource = {
        'default_sort': [('_created', 1)],
    }

    mongo_prefix = MONGO_PREFIX



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
from superdesk.errors import SuperdeskApiError
from superdesk.metadata.item import CONTENT_TYPE, ITEM_TYPE, ITEM_STATE, CONTENT_STATE, PUBLISH_SCHEDULE, EMBARGO

from apps.archive.common import set_sign_off, ITEM_OPERATION
from apps.archive.archive import update_word_count
from superdesk.utc import utcnow

from .common import BasePublishService, BasePublishResource, ITEM_PUBLISH


logger = logging.getLogger(__name__)


class ArchivePublishResource(BasePublishResource):
    def __init__(self, endpoint_name, app, service):
        super().__init__(endpoint_name, app, service, 'publish')


class ArchivePublishService(BasePublishService):
    publish_type = 'publish'
    published_state = 'published'

    def _validate(self, original, updates):
        super()._validate(original, updates)
        if original[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE:
            items = self.package_service.get_residrefs(original)

            if len(items) == 0 and self.publish_type == ITEM_PUBLISH:
                raise SuperdeskApiError.badRequestError("Empty package cannot be published!")

    def on_update(self, updates, original):
        if not original.get('firstpublished'):
            updates.setdefault('firstpublished', utcnow())
        updates[ITEM_OPERATION] = ITEM_PUBLISH
        super().on_update(updates, original)
        set_sign_off(updates, original)
        update_word_count(updates)

    def set_state(self, original, updates):
        """Set the state of the document to schedule if the publish_schedule is specified.

        :param dict original: original document
        :param dict updates: updates related to original document
        """
        updates.setdefault(ITEM_OPERATION, ITEM_PUBLISH)
        if original.get(PUBLISH_SCHEDULE) or updates.get(PUBLISH_SCHEDULE):
            updates[ITEM_STATE] = CONTENT_STATE.SCHEDULED
        elif original.get(EMBARGO) or updates.get(EMBARGO):
            updates[ITEM_STATE] = CONTENT_STATE.PUBLISHED
        else:
            super().set_state(original, updates)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import flask
import logging
import superdesk

from datetime import timedelta
from flask import request, current_app as app
from eve.auth import TokenAuth
from superdesk.resource import Resource
from superdesk.errors import SuperdeskApiError
from superdesk import get_resource_service, get_resource_privileges, get_intrinsic_privileges
from superdesk.utc import utcnow


logger = logging.getLogger(__name__)


class AuthUsersResource(Resource):
    """This resource is for authentication only.

    On users `find_one` never returns a password due to the projection.
    """

    datasource = {'source': 'users'}
    schema = {
        'username': {
            'type': 'string',
        },
        'password': {
            'type': 'string',
        },
        'is_active': {
            'type': 'boolean'
        },
        'is_enabled': {
            'type': 'boolean'
        }
    }
    item_methods = []
    resource_methods = []
    internal_resource = True


class AuthResource(Resource):
    schema = {
        'username': {
            'type': 'string',
            'required': True
        },
        'password': {
            'type': 'string',
            'required': True
        },
        'token': {
            'type': 'string'
        },
        'user': Resource.rel('users', True)
    }

    resource_methods = ['POST']
    item_methods = ['GET', 'DELETE']
    public_methods = ['POST', 'DELETE']
    extra_response_fields = ['user', 'token', 'username']
    datasource = {'source': 'auth'}
    mongo_indexes = {'token': ([('token', 1)], {'background': True})}


superdesk.intrinsic_privilege('auth', method=['DELETE'])


class SuperdeskTokenAuth(TokenAuth):
    """Superdesk Token Auth

    """

    def check_permissions(self, resource, method, user):
        """Checks user permissions.

        1. If there's no user associated with the request or HTTP Method is GET then return True.
        2. Get User's Privileges
        3. Intrinsic Privileges:
            Check if resource has intrinsic privileges.
                If it has then check if HTTP Method is allowed.
                    Return True if `is_authorized()` on the resource service returns True.
                    Otherwise, raise ForbiddenError.
                HTTP Method not allowed continue
            No intrinsic privileges continue
        4. User's Privileges
            Get Resource Privileges and validate it against user's privileges. Return True if validation is successful.
            Otherwise continue.
        5. If method didn't return True, then user is not authorized to perform the requested operation on the resource.
        """

        # Step 1:
        if not user:
            return True

        # Step 2: Get User's Privileges
        get_resource_service('users').set_privileges(user, flask.g.role)

        if method == 'GET':
            return True

        # Step 3: Intrinsic Privileges
        intrinsic_privileges = get_intrinsic_privileges()
        if intrinsic_privileges.get(resource) and method in intrinsic_privileges[resource]:
            service = get_resource_service(resource)
            authorized = service.is_authorized(user_id=str(user.get('_id')), _id=request.view_args.get('_id'))

            if not authorized:
                raise SuperdeskApiError.forbiddenError()

            return authorized

        # Step 4: User's privileges
        privileges = user.get('active_privileges', {})
        resource_privileges = get_resource_privileges(resource).get(method, None)
        if privileges.get(resource_privileges, False):
            return True

        # Step 5:
        raise SuperdeskApiError.forbiddenError()

    def check_auth(self, token, allowed_roles, resource, method):
        """Check if given token is valid.

        If token is valid it updates session and checks permissions.
        """
        auth_service = get_resource_service('auth')
        user_service = get_resource_service('users')
        auth_token = auth_service.find_one(token=token, req=None)
        if auth_token:
            user_id = str(auth_token['user'])
            flask.g.user = user_service.find_one(req=None, _id=user_id)
            flask.g.role = user_service.get_role(flask.g.user)
            flask.g.auth = auth_token
            flask.g.auth_value = auth_token['user']
            if method in ('POST', 'PUT', 'PATCH') or method == 'GET' and not request.args.get('auto'):
                now = utcnow()
                if auth_token[app.config['LAST_UPDATED']] + timedelta(seconds=30) < now:  # update once per 30s max
                    auth_service.update_session({app.config['LAST_UPDATED']: now})
            return self.check_permissions(resource, method, flask.g.user)

    def authorized(self, allowed_roles, resource, method):
        """Ignores auth on home endpoint."""
        if not resource:
            return True
        return super(SuperdeskTokenAuth, self).authorized(allowed_roles, resource, method)

    def authenticate(self):
        """Returns 401 response with CORS headers."""
        raise SuperdeskApiError.unauthorizedError()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2016 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk

from apps.archive.archive import SOURCE as ARCHIVE
from apps.content import push_content_notification
from apps.auth import get_user_id
from superdesk import get_resource_service
from superdesk.errors import SuperdeskApiError, InvalidStateTransitionError
from superdesk.metadata.item import CONTENT_STATE, ITEM_STATE
from superdesk.metadata.packages import RESIDREF
from superdesk.resource import Resource
from superdesk.services import BaseService
from superdesk.workflow import is_workflow_state_transition_valid
from superdesk.utc import utcnow
from apps.packages import PackageService


package_service = PackageService()


class TranslateResource(Resource):
    endpoint_name = 'translate'
    resource_title = endpoint_name

    schema = {
        'guid': {
            'type': 'string',
            'required': True
        },
        'language': {
            'type': 'string',
            'required': True
        },
        'desk': Resource.rel('desks', nullable=True),
    }

    url = 'archive/translate'

    resource_methods = ['POST']
    item_methods = []

    privileges = {'POST': 'translate'}


class TranslateService(BaseService):

    def _translate_item(self, guid, language, task=None, service=None, **kwargs):
        if not service:
            service = ARCHIVE
        archive_service = get_resource_service(service)
        macros_service = get_resource_service('macros')

        item = archive_service.find_one(req=None, _id=guid)
        if not item:
            raise SuperdeskApiError.notFoundError('Fail to found item with guid: %s' % guid)

        if not is_workflow_state_transition_valid('translate', item[ITEM_STATE]):
            raise InvalidStateTransitionError()

        if item.get('language') == language:
            return guid

        if package_service.is_package(item):
            refs = package_service.get_item_refs(item)
            for ref in refs:
                ref[RESIDREF] = self._translate_item(ref[RESIDREF], language,
                                                     service=ref.get('location'),
                                                     task=task)

        macros_service.execute_translation_macro(
            item, item.get('language', None), language)

        item['language'] = language
        item['translated_from'] = guid
        item['versioncreated'] = utcnow()
        if task:
            item['task'] = task

        _id = archive_service.duplicate_item(item, operation='translate')

        if kwargs.get('notify', True):
            push_content_notification([item])

        return _id

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            task = None
            if doc.get('desk'):
                desk = get_resource_service('desks').find_one(req=None, _id=doc['desk']) or {}
                task = dict(desk=desk.get('_id'), stage=desk.get('working_stage'), user=get_user_id())
            ids.append(self._translate_item(doc['guid'], doc['language'], task, **kwargs))
        return ids


superdesk.workflow_action(
    name='translate',
    exclude_states=[CONTENT_STATE.SPIKED, CONTENT_STATE.KILLED],
    privileges=['archive', 'translate']
)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from apps.archive.common import ARCHIVE
from superdesk import get_resource_service
from superdesk.metadata.item import ITEM_TYPE, CONTENT_TYPE, PUBLISH_STATES, \
    ITEM_STATE
from superdesk.resource import Resource

from apps.packages.package_service import PackageService, create_root_group,\
    get_item_ref
from eve.utils import config
from eve.validation import ValidationError
from superdesk.errors import SuperdeskApiError
from superdesk.services import BaseService
from superdesk.metadata.packages import GROUPS, GROUP_ID, REFS, RESIDREF,\
    ROOT_GROUP, ID_REF, PACKAGE_TYPE


class PublishedPackageItemsResource(Resource):
    schema = {
        'package_id': {
            'type': 'string',
            'required': True
        },
        'new_items': {
            'type': 'list',
            'required': True,
            'schema': {
                'type': 'dict',
                'schema': {
                    'group': {'type': 'string'},
                    'item_id': {'type': 'string'}
                }
            }
        }
    }
    datasource = {
        'source': 'archive'
    }
    resource_methods = ['POST']
    privileges = {'POST': ARCHIVE}


class PublishedPackageItemsService(BaseService):
    package_service = PackageService()

    def create(self, docs, **kwargs):
        ids = []
        for doc in docs:
            original = get_resource_service(ARCHIVE).find_one(req=None, _id=doc['package_id'])
            if not original or original[ITEM_TYPE] != CONTENT_TYPE.COMPOSITE:
                raise SuperdeskApiError.badRequestError('Invalid package identifier')
            if original[ITEM_STATE] not in PUBLISH_STATES:
                raise SuperdeskApiError.badRequestError('Package was not published')

            items = {}
            for new_item in doc['new_items']:
                item = get_resource_service(ARCHIVE).find_one(req=None, _id=new_item['item_id'])
                if not item:
                    raise SuperdeskApiError.badRequestError('Invalid item identifier %s' % new_item['item_id'])
                try:
                    self.package_service.check_for_circular_reference(original, new_item['item_id'])
                except ValidationError:
                    raise SuperdeskApiError.badRequestError('Circular reference in item %s', new_item['item_id'])
                items[item[config.ID_FIELD]] = item

            updates = {key: original[key] for key in [config.ID_FIELD, PACKAGE_TYPE, GROUPS]
                       if key in original}
            create_root_group([updates])
            items_refs = []
            for new_item in doc['new_items']:
                items_refs.append(self._set_item_assoc(updates, new_item, items[new_item['item_id']]))
            get_resource_service(ARCHIVE).system_update(original[config.ID_FIELD], updates, original)
            for item_ref in items_refs:
                self.package_service.update_link(updates, item_ref)

            items_published = [new_item[ITEM_STATE] in PUBLISH_STATES for new_item in items.values()]
            if any(items_published):
                get_resource_service('archive_correct').patch(id=doc['package_id'], updates=updates)

            ids.append(original[config.ID_FIELD])
        return ids

    def _set_item_assoc(self, package, new_item, item_doc):
        group = self._get_group(package, new_item['group'])
        for assoc in group[REFS]:
            if assoc.get(RESIDREF) == new_item['item_id']:
                return assoc
        item_ref = get_item_ref(item_doc)
        group[REFS].append(item_ref)
        return item_ref

    def _get_group(self, package, group):
        for package_group in package[GROUPS]:
            if group == package_group[GROUP_ID]:
                return package_group
        self._add_group_in_root(group, package[GROUPS])
        package[GROUPS].append({GROUP_ID: group, REFS: []})
        return package[GROUPS][-1]

    def _add_group_in_root(self, group, groups):
        root_refs = []
        for group_meta in groups:
            if group_meta.get(GROUP_ID) == ROOT_GROUP:
                root_refs = [ref[ID_REF] for ref in group_meta[REFS]]
                if group not in root_refs:
                    group_meta[REFS].append({ID_REF: group})



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import logging
import superdesk
from .service import ContactsService
from .resource import ContactsResource, CONTACTS_PRIVILEDGE, VIEW_CONTACTS

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = 'contacts'
    service = ContactsService(endpoint_name, backend=superdesk.get_backend())
    ContactsResource(endpoint_name, app=app, service=service)
    superdesk.privilege(name=CONTACTS_PRIVILEDGE, label='Contacts Management', description='Manage Contacts')
    superdesk.privilege(name=VIEW_CONTACTS, label='Contacts View Rights', description='View Contacts')



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import os
import ftplib
import logging
import tempfile

from datetime import datetime

from superdesk.io.registry import register_feeding_service
from superdesk.io.feed_parsers import XMLFeedParser
from superdesk.utc import utc
from superdesk.etree import etree
from superdesk.io.feeding_services import FeedingService
from superdesk.errors import IngestFtpError
from superdesk.ftp import ftp_connect
from superdesk.io.commands.update_ingest import LAST_UPDATED

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

logger = logging.getLogger(__name__)
DEFAULT_SUCCESS_PATH = "_PROCESSED"
DEFAULT_FAILURE_PATH = "_ERROR"


class FTPFeedingService(FeedingService):
    """
    Feeding Service class which can read article(s) which exist in a file system and accessible using FTP.
    """

    NAME = 'ftp'
    ERRORS = [IngestFtpError.ftpUnknownParserError().get_error_description(),
              IngestFtpError.ftpError().get_error_description()]

    DATE_FORMAT = '%Y%m%d%H%M%S'
    ALLOWED_EXT_DEFAULT = {'.json', '.xml'}

    def config_from_url(self, url):
        """
        Parse given url into ftp config.

        :param url: url in form `ftp://username:password@host:port/dir`
        """
        url_parts = urlparse(url)
        return {
            'username': url_parts.username,
            'password': url_parts.password,
            'host': url_parts.hostname,
            'path': url_parts.path.lstrip('/'),
        }

    def _test(self, provider):
        config = provider.get('config', {})
        try:
            with ftp_connect(config) as ftp:
                ftp.mlsd()
        except IngestFtpError:
            raise
        except Exception as ex:
            raise IngestFtpError.ftpError(ex, provider)

    def _move(self, ftp, src, dest):
        """Move distant file

        :param ftp: FTP instance to use
        :type ftp: ftplib.FTP
        :param src: source path of the file to move
        :type src: str
        :param dest: dest path of the file to move
        :type dest: str
        """
        try:
            ftp.rename(src, dest)
        except ftplib.all_errors as e:
            logger.warning("Can't move file from {src} to {dest}: {reason}".format(
                src=src,
                dest=dest,
                reason=e))

    def _create_if_missing(self, ftp, path):
        """Check if a dir exists, and create it else

        :param ftp: FTP instance to use
        :type ftp: ftplib.FTP
        :param src: dir path to check
        :type src: str
        """
        base_path = ftp.pwd()
        try:
            ftp.cwd(path)
        except ftplib.all_errors:
            # path probably doesn't exist
            # catching all_errors is a bit overkill,
            # but ftplib doesn't really have precise error
            # for missing directory
            ftp.mkd(path)
        finally:
            ftp.cwd(base_path)

    def _is_allowed(self, filename, allowed_ext):
        """Test if given file is allowed to be ingested."""
        _, ext = os.path.splitext(filename)
        return ext.lower() in allowed_ext

    def _update(self, provider, update):
        config = provider.get('config', {})
        last_updated = provider.get('last_updated')
        registered_parser = self.get_feed_parser(provider)
        try:
            allowed_ext = registered_parser.ALLOWED_EXT
        except AttributeError:
            allowed_ext = self.ALLOWED_EXT_DEFAULT
        crt_last_updated = None
        if config.get('move', False):
            do_move = True
            if not config.get('move_path'):
                logger.debug('missing move_path, default will be used')
            move_dest_path = os.path.join(config.get('path', ''), config.get('move_path') or DEFAULT_SUCCESS_PATH)
            if not config.get('move_path_error'):
                logger.debug('missing move_path_error, default will be used')
            move_dest_path_error = os.path.join(config.get('path', ''),
                                                config.get('move_path_error') or DEFAULT_FAILURE_PATH)
        else:
            do_move = False

        if 'dest_path' not in config:
            config['dest_path'] = tempfile.mkdtemp(prefix='superdesk_ingest_')

        try:
            with ftp_connect(config) as ftp:
                if do_move:
                    try:
                        self._create_if_missing(ftp, move_dest_path)
                        self._create_if_missing(ftp, move_dest_path_error)
                    except ftplib.all_errors as e:
                        logger.warning("Can't create move directory, files will not be moved: {reason}".format(
                            reason=e))
                        do_move = False
                items = []
                for filename, facts in ftp.mlsd():
                    if facts.get('type', '') != 'file':
                        continue
                    try:
                        if not self._is_allowed(filename, allowed_ext):
                            logger.info('ignoring file {filename} because of file extension'.format(filename=filename))
                            continue

                        if last_updated:
                            item_last_updated = datetime.strptime(facts['modify'], self.DATE_FORMAT).replace(tzinfo=utc)
                            if item_last_updated <= last_updated:
                                continue
                            elif not crt_last_updated or item_last_updated > crt_last_updated:
                                crt_last_updated = item_last_updated

                        local_file_path = os.path.join(config['dest_path'], filename)
                        with open(local_file_path, 'wb') as f:
                            try:
                                ftp.retrbinary('RETR %s' % filename, f.write)
                            except ftplib.all_errors as ex:
                                os.remove(local_file_path)
                                raise Exception('Exception retrieving file from FTP server ({filename})'.format(
                                                filename=filename))

                        if isinstance(registered_parser, XMLFeedParser):
                            xml = etree.parse(local_file_path).getroot()
                            parser = self.get_feed_parser(provider, xml)
                            parsed = parser.parse(xml, provider)
                        else:
                            parser = self.get_feed_parser(provider, local_file_path)
                            parsed = parser.parse(local_file_path, provider)

                        if isinstance(parsed, dict):
                            parsed = [parsed]

                        items.append(parsed)
                        if do_move:
                            move_dest_file_path = os.path.join(move_dest_path, filename)
                            self._move(ftp, filename, move_dest_file_path)
                    except Exception as e:
                        logger.error("Error while parsing {filename}: {msg}".format(filename=filename, msg=e))
                        if do_move:
                            move_dest_file_path_error = os.path.join(move_dest_path_error, filename)
                            self._move(ftp, filename, move_dest_file_path_error)
            if crt_last_updated:
                update[LAST_UPDATED] = crt_last_updated
            return items
        except IngestFtpError:
            raise
        except Exception as ex:
            raise IngestFtpError.ftpError(ex, provider)


register_feeding_service(FTPFeedingService.NAME, FTPFeedingService(), FTPFeedingService.ERRORS)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import superdesk
import elasticsearch

from superdesk.utils import get_random_string
from eve_elastic import get_es, get_indices, reindex
from apps.search import SearchService


class RebuildElasticIndex(superdesk.Command):
    """Rebuild the elastic index from existing data.

    It is creating a new index with the same alias as the configured index,
    puts the new mapping and delete the old index.
    """

    option_list = [
        superdesk.Option('--index', '-i', dest='index_name')
    ]

    def run(self, index_name=None):
        # if no index name is passed then use the configured one
        index_name = index_name if index_name else superdesk.app.config['ELASTICSEARCH_INDEX']
        print('Starting index rebuilding for index: {}'.format(index_name))
        if index_name not in self._get_available_indexes():
            raise Exception("Index {} is not configured".format(index_name))
        try:
            es = get_es(superdesk.app.config['ELASTICSEARCH_URL'])
            clone_name = index_name + '-' + get_random_string()
            print('Creating index: ', clone_name)
            superdesk.app.data.elastic.create_index(clone_name, superdesk.app.config['ELASTICSEARCH_SETTINGS'])
            real_name = superdesk.app.data.elastic.get_index_by_alias(clone_name)
            print('Putting mapping for index: ', clone_name)
            superdesk.app.data.elastic.put_mapping(superdesk.app, clone_name)
            print('Starting index rebuilding.')
            reindex(es, index_name, clone_name)
            print('Finished index rebuilding.')
            print('Deleting index: ', index_name)
            get_indices(es).delete(index_name)
            print('Creating alias: ', index_name)
            get_indices(es).put_alias(index=real_name, name=index_name)
            print('Alias created.')
            print('Deleting clone name alias')
            get_indices(es).delete_alias(name=clone_name, index=real_name)
            print('Deleted clone name alias')
        except elasticsearch.exceptions.NotFoundError as nfe:
            print(nfe)
        print('Index {0} rebuilt successfully.'.format(index_name))

    def _get_available_indexes(self):
        return SearchService(None, None).get_available_indexes()


superdesk.command('app:rebuild_elastic_index', RebuildElasticIndex())



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015, 2016, 2017 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
import logging

from superdesk.tests import TestCase
from .remove_expired_items import RemoveExpiredItems
from superdesk import get_resource_service
from datetime import timedelta
from superdesk.utc import utcnow

logger = logging.getLogger(__name__)

utc = utcnow()
expired = utc - timedelta(days=10)
not_expired = utc - timedelta(days=1)
items = [
    {'_id': '1', '_updated': not_expired, 'type': 'text'},  # Single item, not expired

    {'_id': '2', '_updated': expired, 'type': 'text'},  # Single item, expired

    # Evolved from: parent expired, child not expired
    {'_id': '3', '_updated': expired, 'type': 'text'},
    {'_id': '4', '_updated': not_expired, 'type': 'text', 'evolvedfrom': '3', 'ancestors': ['3']},

    # Evolved from: parent expired, child expired
    {'_id': '5', '_updated': expired, 'type': 'text'},
    {'_id': '6', '_updated': expired, 'type': 'text', 'evolvedfrom': '5', 'ancestors': ['5']},

    # Multi-branch evolved from,
    {'_id': '7', '_updated': expired, 'type': 'text'},
    {'_id': '8', '_updated': expired, 'type': 'text', 'evolvedfrom': '7', 'ancestors': ['7']},
    {'_id': '9', '_updated': not_expired, 'type': 'text', 'evolvedfrom': '8', 'ancestors': ['7', '8']},

    # Multi-branch evolved from
    {'_id': '10', '_updated': expired, 'type': 'text'},
    {'_id': '11', '_updated': expired, 'type': 'text', 'evolvedfrom': '10', 'ancestors': ['10']},
    {'_id': '12', '_updated': expired, 'type': 'text', 'evolvedfrom': '11', 'ancestors': ['10', '11']},
]

can_expire = [
    '2',  # Single item, expired
    '5', '6',  # Evolved from, parent expired, child expired
    '10', '11', '12'  # Multi-branch evolved from
]


class RemoveExpiredItemsTest(TestCase):
    def setUp(self):
        self.app.data.insert('items', items)
        self.command = RemoveExpiredItems()
        self.command.expiry_days = 8
        self.now = utcnow()

    def _get_items(self):
        return list(get_resource_service('items').get_from_mongo(req=None, lookup=None))

    def test_remove_expired_items(self):
        self.command._remove_expired_items(self.now, self.command.expiry_days)
        items = self._get_items()

        self.assertEqual(len(items), 6)
        for item in items:
            self.assertNotIn(item['_id'], can_expire)

    def test_get_expired_chain(self):
        items_service = get_resource_service('items')
        for item in self._get_items():
            if self.command._get_expired_chain(items_service, item, self.now):
                self.assertTrue(item['_id'] in can_expire)
            else:
                self.assertFalse(item['_id'] in can_expire)

    def test_run(self):
        self.command.run(self.command.expiry_days)
        items = self._get_items()

        self.assertEqual(len(items), 6)
        for item in items:
            self.assertNotIn(item['_id'], can_expire)

    def test_has_expired(self):
        has_expired = self.command._has_expired(items[1], self.now)
        self.assertTrue(has_expired)

        has_expired = self.command._has_expired(items[0], self.now)
        self.assertFalse(has_expired)

    def test_get_children(self):
        service = get_resource_service('items')
        children = self.command._get_children(service, items[0])
        self.assertEqual(children, [])

        children = [child['_id'] for child in self.command._get_children(service, items[2])]
        self.assertEqual(children, ['4'])

        children = [child['_id'] for child in self.command._get_children(service, items[9])]
        self.assertEqual(children, ['11', '12'])
